from __future__ import print_function,absolute_import,division,unicode_literals
_B=False
_A=None
from .anchor import Anchor
if _B:from typing import Text,Any,Dict,List
__all__=['ScalarBoolean']
class ScalarBoolean(int):
	def __new__(D,*E,**A):
		B=A.pop('anchor',_A);C=int.__new__(D,*E,**A)
		if B is not _A:C.yaml_set_anchor(B,always_dump=True)
		return C
	@property
	def anchor(self):
		A=self
		if not hasattr(A,Anchor.attrib):setattr(A,Anchor.attrib,Anchor())
		return getattr(A,Anchor.attrib)
	def yaml_anchor(A,any=_B):
		if not hasattr(A,Anchor.attrib):return _A
		if any or A.anchor.always_dump:return A.anchor
		return _A
	def yaml_set_anchor(A,value,always_dump=_B):A.anchor.value=value;A.anchor.always_dump=always_dump


from __future__ import absolute_import,print_function,unicode_literals
_E='.env'
_D='always'
_C=True
_B=False
_A=None
import io,logging,os,re,shutil,sys,tempfile
from collections import OrderedDict
from contextlib import contextmanager
from .compat import IS_TYPE_CHECKING,PY2,StringIO,to_env
from .parser import Binding,parse_stream
logger=logging.getLogger(__name__)
if IS_TYPE_CHECKING:
	from typing import Dict,Iterator,Match,Optional,Pattern,Union,Text,IO,Tuple
	if sys.version_info>=(3,6):_PathLike=os.PathLike
	else:_PathLike=Text
	if sys.version_info>=(3,0):_StringIO=StringIO
	else:_StringIO=StringIO[Text]
__posix_variable=re.compile('\n    \\$\\{\n        (?P<name>[^\\}:]*)\n        (?::-\n            (?P<default>[^\\}]*)\n        )?\n    \\}\n    ',re.VERBOSE)
def with_warn_for_invalid_lines(mappings):
	for A in mappings:
		if A.error:logger.warning('Python-dotenv could not parse statement starting at line %s',A.original.line)
		yield A
class DotEnv:
	def __init__(A,dotenv_path,verbose=_B,encoding=_A,interpolate=_C):A.dotenv_path=dotenv_path;A._dict=_A;A.verbose=verbose;A.encoding=encoding;A.interpolate=interpolate
	@contextmanager
	def _get_stream(self):
		A=self
		if isinstance(A.dotenv_path,StringIO):yield A.dotenv_path
		elif os.path.isfile(A.dotenv_path):
			with io.open(A.dotenv_path,encoding=A.encoding)as B:yield B
		else:
			if A.verbose:logger.info('Python-dotenv could not find configuration file %s.',A.dotenv_path or _E)
			yield StringIO('')
	def dict(A):
		if A._dict:return A._dict
		B=OrderedDict(A.parse());A._dict=resolve_nested_variables(B)if A.interpolate else B;return A._dict
	def parse(B):
		with B._get_stream()as C:
			for A in with_warn_for_invalid_lines(parse_stream(C)):
				if A.key is not _A:yield(A.key,A.value)
	def set_as_environment_variables(C,override=_B):
		for (A,B) in C.dict().items():
			if A in os.environ and not override:continue
			if B is not _A:os.environ[to_env(A)]=to_env(B)
		return _C
	def get(A,key):
		B=key;C=A.dict()
		if B in C:return C[B]
		if A.verbose:logger.warning('Key %s not found in %s.',B,A.dotenv_path)
		return _A
def get_key(dotenv_path,key_to_get):return DotEnv(dotenv_path,verbose=_C).get(key_to_get)
@contextmanager
def rewrite(path):
	try:
		with tempfile.NamedTemporaryFile(mode='w+',delete=_B)as A:
			with io.open(path)as B:yield(B,A)
	except BaseException:
		if os.path.isfile(A.name):os.unlink(A.name)
		raise
	else:shutil.move(A.name,path)
def set_key(dotenv_path,key_to_set,value_to_set,quote_mode=_D):
	K='"';E=quote_mode;C=dotenv_path;B=key_to_set;A=value_to_set;A=A.strip("'").strip(K)
	if not os.path.exists(C):logger.warning("Can't write to %s - it doesn't exist.",C);return _A,B,A
	if' 'in A:E=_D
	if E==_D:F='"{}"'.format(A.replace(K,'\\"'))
	else:F=A
	G='{}={}\n'.format(B,F)
	with rewrite(C)as(J,D):
		H=_B
		for I in with_warn_for_invalid_lines(parse_stream(J)):
			if I.key==B:D.write(G);H=_C
			else:D.write(I.original.string)
		if not H:D.write(G)
	return _C,B,A
def unset_key(dotenv_path,key_to_unset,quote_mode=_D):
	B=dotenv_path;A=key_to_unset
	if not os.path.exists(B):logger.warning("Can't delete from %s - it doesn't exist.",B);return _A,A
	C=_B
	with rewrite(B)as(E,F):
		for D in with_warn_for_invalid_lines(parse_stream(E)):
			if D.key==A:C=_C
			else:F.write(D.original.string)
	if not C:logger.warning("Key %s not removed from %s - key doesn't exist.",A,B);return _A,A
	return C,A
def resolve_nested_variables(values):
	def C(name,default):A=default;A=A if A is not _A else'';C=os.getenv(name,B.get(name,A));return C
	def D(match):A=match.groupdict();return C(name=A['name'],default=A['default'])
	B={}
	for (E,A) in values.items():B[E]=__posix_variable.sub(D,A)if A is not _A else _A
	return B
def _walk_to_root(path):
	A=path
	if not os.path.exists(A):raise IOError('Starting path not found')
	if os.path.isfile(A):A=os.path.dirname(A)
	C=_A;B=os.path.abspath(A)
	while C!=B:yield B;D=os.path.abspath(os.path.join(B,os.path.pardir));C,B=B,D
def find_dotenv(filename=_E,raise_error_if_not_found=_B,usecwd=_B):
	H='.py'
	def E():B='__file__';A=__import__('__main__',_A,_A,fromlist=[B]);return not hasattr(A,B)
	if usecwd or E()or getattr(sys,'frozen',_B):B=os.getcwd()
	else:
		A=sys._getframe()
		if PY2 and not __file__.endswith(H):C=__file__.rsplit('.',1)[0]+H
		else:C=__file__
		while A.f_code.co_filename==C:assert A.f_back is not _A;A=A.f_back
		F=A.f_code.co_filename;B=os.path.dirname(os.path.abspath(F))
	for G in _walk_to_root(B):
		D=os.path.join(G,filename)
		if os.path.isfile(D):return D
	if raise_error_if_not_found:raise IOError('File not found')
	return''
def load_dotenv(dotenv_path=_A,stream=_A,verbose=_B,override=_B,interpolate=_C,**A):B=dotenv_path or stream or find_dotenv();return DotEnv(B,verbose=verbose,interpolate=interpolate,**A).set_as_environment_variables(override=override)
def dotenv_values(dotenv_path=_A,stream=_A,verbose=_B,interpolate=_C,**A):B=dotenv_path or stream or find_dotenv();return DotEnv(B,verbose=verbose,interpolate=interpolate,**A).dict()


_Z='keys'
_Y='box_settings'
_X='default_box_attr'
_W='Box is frozen'
_V='modify_tuples_box'
_U='box_safe_prefix'
_T='default_box_none_transform'
_S='__created'
_R='box_dots'
_Q='box_duplicates'
_P='ignore'
_O='.'
_N='strict'
_M='box_recast'
_L='box_intact_types'
_K='default_box'
_J='_'
_I='utf-8'
_H='_box_config'
_G=True
_F='camel_killer_box'
_E='conversion_box'
_D='frozen_box'
_C='__safe_keys'
_B=False
_A=None
import copy,re,string,warnings
from collections.abc import Iterable,Mapping,Callable
from keyword import kwlist
from pathlib import Path
from typing import Any,Union,Tuple,List,Dict
from dynaconf.vendor import box
from .converters import _to_json,_from_json,_from_toml,_to_toml,_from_yaml,_to_yaml,BOX_PARAMETERS
from .exceptions import BoxError,BoxKeyError,BoxTypeError,BoxValueError,BoxWarning
__all__=['Box']
_first_cap_re=re.compile('(.)([A-Z][a-z]+)')
_all_cap_re=re.compile('([a-z0-9])([A-Z])')
_list_pos_re=re.compile('\\[(\\d+)\\]')
NO_DEFAULT=object()
def _camel_killer(attr):D='\\1_\\2';A=attr;A=str(A);B=_first_cap_re.sub(D,A);C=_all_cap_re.sub(D,B);return re.sub(' *_+',_J,C.lower())
def _recursive_tuples(iterable,box_class,recreate_tuples=_B,**E):
	D=recreate_tuples;C=box_class;B=[]
	for A in iterable:
		if isinstance(A,dict):B.append(C(A,**E))
		elif isinstance(A,list)or D and isinstance(A,tuple):B.append(_recursive_tuples(A,C,D,**E))
		else:B.append(A)
	return tuple(B)
def _parse_box_dots(item):
	A=item
	for (B,C) in enumerate(A):
		if C=='[':return A[:B],A[B:]
		elif C==_O:return A[:B],A[B+1:]
	raise BoxError('Could not split box dots properly')
def _get_box_config():return{_S:_B,_C:{}}
class Box(dict):
	_protected_keys=['to_dict','to_json','to_yaml','from_yaml','from_json','from_toml','to_toml','merge_update']+[A for A in dir({})if not A.startswith(_J)]
	def __new__(A,*D,box_settings=_A,default_box=_B,default_box_attr=NO_DEFAULT,default_box_none_transform=_G,frozen_box=_B,camel_killer_box=_B,conversion_box=_G,modify_tuples_box=_B,box_safe_prefix='x',box_duplicates=_P,box_intact_types=(),box_recast=_A,box_dots=_B,**E):C=default_box_attr;B=super(Box,A).__new__(A,*D,**E);B._box_config=_get_box_config();B._box_config.update({_K:default_box,_X:A.__class__ if C is NO_DEFAULT else C,_T:default_box_none_transform,_E:conversion_box,_U:box_safe_prefix,_D:frozen_box,_F:camel_killer_box,_V:modify_tuples_box,_Q:box_duplicates,_L:tuple(box_intact_types),_M:box_recast,_R:box_dots,_Y:box_settings or{}});return B
	def __init__(A,*B,box_settings=_A,default_box=_B,default_box_attr=NO_DEFAULT,default_box_none_transform=_G,frozen_box=_B,camel_killer_box=_B,conversion_box=_G,modify_tuples_box=_B,box_safe_prefix='x',box_duplicates=_P,box_intact_types=(),box_recast=_A,box_dots=_B,**F):
		E=default_box_attr;super().__init__();A._box_config=_get_box_config();A._box_config.update({_K:default_box,_X:A.__class__ if E is NO_DEFAULT else E,_T:default_box_none_transform,_E:conversion_box,_U:box_safe_prefix,_D:frozen_box,_F:camel_killer_box,_V:modify_tuples_box,_Q:box_duplicates,_L:tuple(box_intact_types),_M:box_recast,_R:box_dots,_Y:box_settings or{}})
		if not A._box_config[_E]and A._box_config[_Q]!=_P:raise BoxError('box_duplicates are only for conversion_boxes')
		if len(B)==1:
			if isinstance(B[0],str):raise BoxValueError('Cannot extrapolate Box from string')
			if isinstance(B[0],Mapping):
				for (D,C) in B[0].items():
					if C is B[0]:C=A
					if C is _A and A._box_config[_K]and A._box_config[_T]:continue
					A.__setitem__(D,C)
			elif isinstance(B[0],Iterable):
				for (D,C) in B[0]:A.__setitem__(D,C)
			else:raise BoxValueError('First argument must be mapping or iterable')
		elif B:raise BoxTypeError(f"Box expected at most 1 argument, got {len(B)}")
		for (D,C) in F.items():
			if B and isinstance(B[0],Mapping)and C is B[0]:C=A
			A.__setitem__(D,C)
		A._box_config[_S]=_G
	def __add__(C,other):
		A=other;B=C.copy()
		if not isinstance(A,dict):raise BoxTypeError(f"Box can only merge two boxes or a box and a dictionary.")
		B.merge_update(A);return B
	def __hash__(A):
		if A._box_config[_D]:
			B=54321
			for C in A.items():B^=hash(C)
			return B
		raise BoxTypeError('unhashable type: "Box"')
	def __dir__(B):
		D=string.ascii_letters+string.digits+_J;C=set(super().__dir__())
		for A in B.keys():
			A=str(A)
			if' 'not in A and A[0]not in string.digits and A not in kwlist:
				for E in A:
					if E not in D:break
				else:C.add(A)
		for A in B.keys():
			if A not in C:
				if B._box_config[_E]:
					A=B._safe_attr(A)
					if A:C.add(A)
		return list(C)
	def get(B,key,default=NO_DEFAULT):
		C=key;A=default
		if C not in B:
			if A is NO_DEFAULT:
				if B._box_config[_K]and B._box_config[_T]:return B.__get_default(C)
				else:return _A
			if isinstance(A,dict)and not isinstance(A,Box):return Box(A,box_settings=B._box_config.get(_Y))
			if isinstance(A,list)and not isinstance(A,box.BoxList):return box.BoxList(A)
			return A
		return B[C]
	def copy(A):return Box(super().copy(),**A.__box_config())
	def __copy__(A):return Box(super().copy(),**A.__box_config())
	def __deepcopy__(A,memodict=_A):
		B=memodict;E=A._box_config[_D];D=A.__box_config();D[_D]=_B;C=A.__class__(**D);B=B or{};B[id(A)]=C
		for (F,G) in A.items():C[copy.deepcopy(F,B)]=copy.deepcopy(G,B)
		C._box_config[_D]=E;return C
	def __setstate__(A,state):B=state;A._box_config=B[_H];A.__dict__.update(B)
	def keys(A):return super().keys()
	def values(A):return[A[B]for B in A.keys()]
	def items(A):return[(B,A[B])for B in A.keys()]
	def __get_default(B,item):
		A=B._box_config[_X]
		if A in(B.__class__,dict):C=B.__class__(**B.__box_config())
		elif isinstance(A,dict):C=B.__class__(**B.__box_config(),**A)
		elif isinstance(A,list):C=box.BoxList(**B.__box_config())
		elif isinstance(A,Callable):C=A()
		elif hasattr(A,'copy'):C=A.copy()
		else:C=A
		B.__convert_and_store(item,C);return C
	def __box_config(C):
		A={}
		for (B,D) in C._box_config.copy().items():
			if not B.startswith('__'):A[B]=D
		return A
	def __recast(A,item,value):
		C=value;B=item
		if A._box_config[_M]and B in A._box_config[_M]:
			try:return A._box_config[_M][B](C)
			except ValueError:raise BoxValueError(f"Cannot convert {C} to {A._box_config[_M][B]}") from _A
		return C
	def __convert_and_store(B,item,value):
		C=item;A=value
		if B._box_config[_E]:D=B._safe_attr(C);B._box_config[_C][D]=C
		if isinstance(A,(int,float,str,bytes,bytearray,bool,complex,set,frozenset)):return super().__setitem__(C,A)
		if B._box_config[_L]and isinstance(A,B._box_config[_L]):return super().__setitem__(C,A)
		if isinstance(A,dict)and not isinstance(A,Box):A=B.__class__(A,**B.__box_config())
		elif isinstance(A,list)and not isinstance(A,box.BoxList):
			if B._box_config[_D]:A=_recursive_tuples(A,B.__class__,recreate_tuples=B._box_config[_V],**B.__box_config())
			else:A=box.BoxList(A,box_class=B.__class__,**B.__box_config())
		elif B._box_config[_V]and isinstance(A,tuple):A=_recursive_tuples(A,B.__class__,recreate_tuples=_G,**B.__box_config())
		super().__setitem__(C,A)
	def __getitem__(B,item,_ignore_default=_B):
		A=item
		try:return super().__getitem__(A)
		except KeyError as E:
			if A==_H:raise BoxKeyError('_box_config should only exist as an attribute and is never defaulted') from _A
			if B._box_config[_R]and isinstance(A,str)and(_O in A or'['in A):
				C,F=_parse_box_dots(A)
				if C in B.keys():
					if hasattr(B[C],'__getitem__'):return B[C][F]
			if B._box_config[_F]and isinstance(A,str):
				D=_camel_killer(A)
				if D in B.keys():return super().__getitem__(D)
			if B._box_config[_K]and not _ignore_default:return B.__get_default(A)
			raise BoxKeyError(str(E)) from _A
	def __getattr__(A,item):
		B=item
		try:
			try:C=A.__getitem__(B,_ignore_default=_G)
			except KeyError:C=object.__getattribute__(A,B)
		except AttributeError as E:
			if B=='__getstate__':raise BoxKeyError(B) from _A
			if B==_H:raise BoxError('_box_config key must exist') from _A
			if A._box_config[_E]:
				D=A._safe_attr(B)
				if D in A._box_config[_C]:return A.__getitem__(A._box_config[_C][D])
			if A._box_config[_K]:return A.__get_default(B)
			raise BoxKeyError(str(E)) from _A
		return C
	def __setitem__(A,key,value):
		C=value;B=key
		if B!=_H and A._box_config[_S]and A._box_config[_D]:raise BoxError(_W)
		if A._box_config[_R]and isinstance(B,str)and _O in B:
			D,E=_parse_box_dots(B)
			if D in A.keys():
				if hasattr(A[D],'__setitem__'):return A[D].__setitem__(E,C)
		C=A.__recast(B,C)
		if B not in A.keys()and A._box_config[_F]:
			if A._box_config[_F]and isinstance(B,str):B=_camel_killer(B)
		if A._box_config[_E]and A._box_config[_Q]!=_P:A._conversion_checks(B)
		A.__convert_and_store(B,C)
	def __setattr__(A,key,value):
		C=value;B=key
		if B!=_H and A._box_config[_D]and A._box_config[_S]:raise BoxError(_W)
		if B in A._protected_keys:raise BoxKeyError(f'Key name "{B}" is protected')
		if B==_H:return object.__setattr__(A,B,C)
		C=A.__recast(B,C);D=A._safe_attr(B)
		if D in A._box_config[_C]:B=A._box_config[_C][D]
		A.__setitem__(B,C)
	def __delitem__(A,key):
		B=key
		if A._box_config[_D]:raise BoxError(_W)
		if B not in A.keys()and A._box_config[_R]and isinstance(B,str)and _O in B:
			C,E=B.split(_O,1)
			if C in A.keys()and isinstance(A[C],dict):return A[C].__delitem__(E)
		if B not in A.keys()and A._box_config[_F]:
			if A._box_config[_F]and isinstance(B,str):
				for D in A:
					if _camel_killer(B)==D:B=D;break
		super().__delitem__(B)
	def __delattr__(A,item):
		B=item
		if A._box_config[_D]:raise BoxError(_W)
		if B==_H:raise BoxError('"_box_config" is protected')
		if B in A._protected_keys:raise BoxKeyError(f'Key name "{B}" is protected')
		try:A.__delitem__(B)
		except KeyError as D:
			if A._box_config[_E]:
				C=A._safe_attr(B)
				if C in A._box_config[_C]:A.__delitem__(A._box_config[_C][C]);del A._box_config[_C][C];return
			raise BoxKeyError(D)
	def pop(B,key,*C):
		A=key
		if C:
			if len(C)!=1:raise BoxError('pop() takes only one optional argument "default"')
			try:D=B[A]
			except KeyError:return C[0]
			else:del B[A];return D
		try:D=B[A]
		except KeyError:raise BoxKeyError('{0}'.format(A)) from _A
		else:del B[A];return D
	def clear(A):super().clear();A._box_config[_C].clear()
	def popitem(A):
		try:B=next(A.__iter__())
		except StopIteration:raise BoxKeyError('Empty box') from _A
		return B,A.pop(B)
	def __repr__(A):return f"<Box: {A.to_dict()}>"
	def __str__(A):return str(A.to_dict())
	def __iter__(A):
		for B in A.keys():yield B
	def __reversed__(A):
		for B in reversed(list(A.keys())):yield B
	def to_dict(D):
		A=dict(D)
		for (C,B) in A.items():
			if B is D:A[C]=A
			elif isinstance(B,Box):A[C]=B.to_dict()
			elif isinstance(B,box.BoxList):A[C]=B.to_list()
		return A
	def update(C,__m=_A,**D):
		B=__m
		if B:
			if hasattr(B,_Z):
				for A in B:C.__convert_and_store(A,B[A])
			else:
				for (A,E) in B:C.__convert_and_store(A,E)
		for A in D:C.__convert_and_store(A,D[A])
	def merge_update(A,__m=_A,**E):
		C=__m
		def D(k,v):
			B=A._box_config[_L]and isinstance(v,A._box_config[_L])
			if isinstance(v,dict)and not B:
				v=A.__class__(v,**A.__box_config())
				if k in A and isinstance(A[k],dict):
					if isinstance(A[k],Box):A[k].merge_update(v)
					else:A[k].update(v)
					return
			if isinstance(v,list)and not B:v=box.BoxList(v,**A.__box_config())
			A.__setitem__(k,v)
		if C:
			if hasattr(C,_Z):
				for B in C:D(B,C[B])
			else:
				for (B,F) in C:D(B,F)
		for B in E:D(B,E[B])
	def setdefault(B,item,default=_A):
		C=item;A=default
		if C in B:return B[C]
		if isinstance(A,dict):A=B.__class__(A,**B.__box_config())
		if isinstance(A,list):A=box.BoxList(A,box_class=B.__class__,**B.__box_config())
		B[C]=A;return A
	def _safe_attr(C,attr):
		B=attr;G=string.ascii_letters+string.digits+_J
		if isinstance(B,tuple):B=_J.join([str(A)for A in B])
		B=B.decode(_I,_P)if isinstance(B,bytes)else str(B)
		if C.__box_config()[_F]:B=_camel_killer(B)
		A=[];D=0
		for (E,F) in enumerate(B):
			if F in G:D=E;A.append(F)
			elif not A:continue
			elif D==E-1:A.append(_J)
		A=''.join(A)[:D+1]
		try:int(A[0])
		except (ValueError,IndexError):pass
		else:A=f"{C.__box_config()[_U]}{A}"
		if A in kwlist:A=f"{C.__box_config()[_U]}{A}"
		return A
	def _conversion_checks(A,item):
		B=A._safe_attr(item)
		if B in A._box_config[_C]:
			C=[f"{item}({B})",f"{A._box_config[_C][B]}({B})"]
			if A._box_config[_Q].startswith('warn'):warnings.warn(f"Duplicate conversion attributes exist: {C}",BoxWarning)
			else:raise BoxError(f"Duplicate conversion attributes exist: {C}")
	def to_json(A,filename=_A,encoding=_I,errors=_N,**B):return _to_json(A.to_dict(),filename=filename,encoding=encoding,errors=errors,**B)
	@classmethod
	def from_json(E,json_string=_A,filename=_A,encoding=_I,errors=_N,**A):
		D={}
		for B in A.copy():
			if B in BOX_PARAMETERS:D[B]=A.pop(B)
		C=_from_json(json_string,filename=filename,encoding=encoding,errors=errors,**A)
		if not isinstance(C,dict):raise BoxError(f"json data not returned as a dictionary, but rather a {type(C).__name__}")
		return E(C,**D)
	def to_yaml(A,filename=_A,default_flow_style=_B,encoding=_I,errors=_N,**B):return _to_yaml(A.to_dict(),filename=filename,default_flow_style=default_flow_style,encoding=encoding,errors=errors,**B)
	@classmethod
	def from_yaml(E,yaml_string=_A,filename=_A,encoding=_I,errors=_N,**A):
		D={}
		for B in A.copy():
			if B in BOX_PARAMETERS:D[B]=A.pop(B)
		C=_from_yaml(yaml_string=yaml_string,filename=filename,encoding=encoding,errors=errors,**A)
		if not isinstance(C,dict):raise BoxError(f"yaml data not returned as a dictionary but rather a {type(C).__name__}")
		return E(C,**D)
	def to_toml(A,filename=_A,encoding=_I,errors=_N):return _to_toml(A.to_dict(),filename=filename,encoding=encoding,errors=errors)
	@classmethod
	def from_toml(D,toml_string=_A,filename=_A,encoding=_I,errors=_N,**B):
		C={}
		for A in B.copy():
			if A in BOX_PARAMETERS:C[A]=B.pop(A)
		E=_from_toml(toml_string=toml_string,filename=filename,encoding=encoding,errors=errors);return D(E,**C)


_D=False
_C='append'
_B='store'
_A=None
import re
from collections import deque
from .exceptions import BadArgumentUsage
from .exceptions import BadOptionUsage
from .exceptions import NoSuchOption
from .exceptions import UsageError
def _unpack_args(args,nargs_spec):
	D=nargs_spec;C=args;C=deque(C);D=deque(D);A=[];B=_A
	def F(c):
		try:
			if B is _A:return c.popleft()
			else:return c.pop()
		except IndexError:return _A
	while D:
		E=F(D)
		if E==1:A.append(F(C))
		elif E>1:
			G=[F(C)for A in range(E)]
			if B is not _A:G.reverse()
			A.append(tuple(G))
		elif E<0:
			if B is not _A:raise TypeError('Cannot have two nargs < 0')
			B=len(A);A.append(_A)
	if B is not _A:A[B]=tuple(C);C=[];A[B+1:]=reversed(A[B+1:])
	return tuple(A),list(C)
def _error_opt_args(nargs,opt):
	B=nargs;A=opt
	if B==1:raise BadOptionUsage(A,f"{A} option requires an argument")
	raise BadOptionUsage(A,f"{A} option requires {B} arguments")
def split_opt(opt):
	A=opt;B=A[:1]
	if B.isalnum():return'',A
	if A[1:2]==B:return A[:2],A[2:]
	return B,A[1:]
def normalize_opt(opt,ctx):
	B=ctx;A=opt
	if B is _A or B.token_normalize_func is _A:return A
	C,A=split_opt(A);return f"{C}{B.token_normalize_func(A)}"
def split_arg_string(string):
	B=string;C=[]
	for D in re.finditer('(\'([^\'\\\\]*(?:\\\\.[^\'\\\\]*)*)\'|\\"([^\\"\\\\]*(?:\\\\.[^\\"\\\\]*)*)\\"|\\S+)\\s*',B,re.S):
		A=D.group().strip()
		if A[:1]==A[-1:]and A[:1]in'"\'':A=A[1:-1].encode('ascii','backslashreplace').decode('unicode-escape')
		try:A=type(B)(A)
		except UnicodeError:pass
		C.append(A)
	return C
class Option:
	def __init__(A,opts,dest,action=_A,nargs=1,const=_A,obj=_A):
		D=action;A._short_opts=[];A._long_opts=[];A.prefixes=set()
		for B in opts:
			C,E=split_opt(B)
			if not C:raise ValueError(f"Invalid start character for option ({B})")
			A.prefixes.add(C[0])
			if len(C)==1 and len(E)==1:A._short_opts.append(B)
			else:A._long_opts.append(B);A.prefixes.add(C)
		if D is _A:D=_B
		A.dest=dest;A.action=D;A.nargs=nargs;A.const=const;A.obj=obj
	@property
	def takes_value(self):return self.action in(_B,_C)
	def process(A,value,state):
		C=value;B=state
		if A.action==_B:B.opts[A.dest]=C
		elif A.action=='store_const':B.opts[A.dest]=A.const
		elif A.action==_C:B.opts.setdefault(A.dest,[]).append(C)
		elif A.action=='append_const':B.opts.setdefault(A.dest,[]).append(A.const)
		elif A.action=='count':B.opts[A.dest]=B.opts.get(A.dest,0)+1
		else:raise ValueError(f"unknown action '{A.action}'")
		B.order.append(A.obj)
class Argument:
	def __init__(A,dest,nargs=1,obj=_A):A.dest=dest;A.nargs=nargs;A.obj=obj
	def process(A,value,state):
		C=state;B=value
		if A.nargs>1:
			D=sum((1 for A in B if A is _A))
			if D==len(B):B=_A
			elif D!=0:raise BadArgumentUsage(f"argument {A.dest} takes {A.nargs} values")
		C.opts[A.dest]=B;C.order.append(A.obj)
class ParsingState:
	def __init__(A,rargs):A.opts={};A.largs=[];A.rargs=rargs;A.order=[]
class OptionParser:
	def __init__(A,ctx=_A):
		B=ctx;A.ctx=B;A.allow_interspersed_args=True;A.ignore_unknown_options=_D
		if B is not _A:A.allow_interspersed_args=B.allow_interspersed_args;A.ignore_unknown_options=B.ignore_unknown_options
		A._short_opt={};A._long_opt={};A._opt_prefixes={'-','--'};A._args=[]
	def add_option(B,opts,dest,action=_A,nargs=1,const=_A,obj=_A):
		D=obj;C=opts
		if D is _A:D=dest
		C=[normalize_opt(A,B.ctx)for A in C];A=Option(C,dest,action=action,nargs=nargs,const=const,obj=D);B._opt_prefixes.update(A.prefixes)
		for E in A._short_opts:B._short_opt[E]=A
		for E in A._long_opts:B._long_opt[E]=A
	def add_argument(B,dest,nargs=1,obj=_A):
		A=obj
		if A is _A:A=dest
		B._args.append(Argument(dest=dest,nargs=nargs,obj=A))
	def parse_args(B,args):
		A=ParsingState(args)
		try:B._process_args_for_options(A);B._process_args_for_args(A)
		except UsageError:
			if B.ctx is _A or not B.ctx.resilient_parsing:raise
		return A.opts,A.largs,A.order
	def _process_args_for_args(B,state):
		A=state;C,D=_unpack_args(A.largs+A.rargs,[A.nargs for A in B._args])
		for (E,F) in enumerate(B._args):F.process(C[E],A)
		A.largs=D;A.rargs=[]
	def _process_args_for_options(C,state):
		B=state
		while B.rargs:
			A=B.rargs.pop(0);D=len(A)
			if A=='--':return
			elif A[:1]in C._opt_prefixes and D>1:C._process_opts(A,B)
			elif C.allow_interspersed_args:B.largs.append(A)
			else:B.rargs.insert(0,A);return
	def _match_long_opt(D,opt,explicit_value,state):
		E=explicit_value;B=state;A=opt
		if A not in D._long_opt:H=[B for B in D._long_opt if B.startswith(A)];raise NoSuchOption(A,possibilities=H,ctx=D.ctx)
		F=D._long_opt[A]
		if F.takes_value:
			if E is not _A:B.rargs.insert(0,E)
			C=F.nargs
			if len(B.rargs)<C:_error_opt_args(C,A)
			elif C==1:G=B.rargs.pop(0)
			else:G=tuple(B.rargs[:C]);del B.rargs[:C]
		elif E is not _A:raise BadOptionUsage(A,f"{A} option does not take a value")
		else:G=_A
		F.process(G,B)
	def _match_short_opt(B,arg,state):
		D=arg;A=state;J=_D;F=1;K=D[0];G=[]
		for L in D[1:]:
			H=normalize_opt(f"{K}{L}",B.ctx);E=B._short_opt.get(H);F+=1
			if not E:
				if B.ignore_unknown_options:G.append(L);continue
				raise NoSuchOption(H,ctx=B.ctx)
			if E.takes_value:
				if F<len(D):A.rargs.insert(0,D[F:]);J=True
				C=E.nargs
				if len(A.rargs)<C:_error_opt_args(C,H)
				elif C==1:I=A.rargs.pop(0)
				else:I=tuple(A.rargs[:C]);del A.rargs[:C]
			else:I=_A
			E.process(I,A)
			if J:break
		if B.ignore_unknown_options and G:A.largs.append(f"{K}{''.join(G)}")
	def _process_opts(B,arg,state):
		G='=';C=state;A=arg;D=_A
		if G in A:E,D=A.split(G,1)
		else:E=A
		F=normalize_opt(E,B.ctx)
		try:B._match_long_opt(F,D,C)
		except NoSuchOption:
			if A[:2]not in B._opt_prefixes:return B._match_short_opt(A,C)
			if not B.ignore_unknown_options:raise
			C.largs.append(A)


# WARNING: THIS EXTENSION IS DEPRECATED in favor of django_dynaconf_v2
# Read more on how to integrate with django on
# https://dynaconf.readthedocs.io/en/latest/guides/django.html
import os

from django.conf import settings as django_settings

from dynaconf import LazySettings

dj = {}

for key in dir(django_settings):
    if key.isupper() and key != "SETTINGS_MODULE":
        dj[key] = getattr(django_settings, key, None)
    dj["ORIGINAL_SETTINGS_MODULE"] = django_settings.SETTINGS_MODULE

dj.setdefault("ENVVAR_PREFIX", "DJANGO")
env_prefix = f"{dj['ENVVAR_PREFIX']}_ENV"  # DJANGO_ENV
dj.setdefault("ENV", os.environ.get(env_prefix, "DEVELOPMENT").upper())

settings = LazySettings(**dj)



from __future__ import print_function,absolute_import,division,unicode_literals
_B='yaml'
_A=False
if _A:from typing import Dict,Any
_package_data=dict(full_package_name='ruamel.yaml',version_info=(0,16,10),__version__='0.16.10',author='Anthon van der Neut',author_email='a.van.der.neut@ruamel.eu',description='ruamel.yaml is a YAML parser/emitter that supports roundtrip preservation of comments, seq/map flow style, and map key order',entry_points=None,since=2014,extras_require={':platform_python_implementation=="CPython" and python_version<="2.7"':['ruamel.ordereddict'],':platform_python_implementation=="CPython" and python_version<"3.9"':['ruamel.yaml.clib>=0.1.2'],'jinja2':['ruamel.yaml.jinja2>=0.2'],'docs':['ryd']},classifiers=['Programming Language :: Python :: 2.7','Programming Language :: Python :: 3.5','Programming Language :: Python :: 3.6','Programming Language :: Python :: 3.7','Programming Language :: Python :: 3.8','Programming Language :: Python :: Implementation :: CPython','Programming Language :: Python :: Implementation :: PyPy','Programming Language :: Python :: Implementation :: Jython','Topic :: Software Development :: Libraries :: Python Modules','Topic :: Text Processing :: Markup','Typing :: Typed'],keywords='yaml 1.2 parser round-trip preserve quotes order config',read_the_docs=_B,supported=[(2,7),(3,5)],tox=dict(env='*',deps='ruamel.std.pathlib',fl8excl='_test/lib'),universal=True,rtfd=_B)
version_info=_package_data['version_info']
__version__=_package_data['__version__']
try:from .cyaml import *;__with_libyaml__=True
except (ImportError,ValueError):__with_libyaml__=_A
from dynaconf.vendor.ruamel.yaml.main import *


from dynaconf.contrib.django_dynaconf_v2 import DjangoDynaconf  # noqa
from dynaconf.contrib.flask_dynaconf import DynaconfConfig  # noqa
from dynaconf.contrib.flask_dynaconf import FlaskDynaconf  # noqa



from __future__ import absolute_import,print_function
_B='typ'
_A=None
import warnings
from .error import MarkedYAMLError,ReusedAnchorWarning
from .compat import utf8,nprint,nprintf
from .events import StreamStartEvent,StreamEndEvent,MappingStartEvent,MappingEndEvent,SequenceStartEvent,SequenceEndEvent,AliasEvent,ScalarEvent
from .nodes import MappingNode,ScalarNode,SequenceNode
if False:from typing import Any,Dict,Optional,List
__all__=['Composer','ComposerError']
class ComposerError(MarkedYAMLError):0
class Composer:
	def __init__(A,loader=_A):
		A.loader=loader
		if A.loader is not _A and getattr(A.loader,'_composer',_A)is _A:A.loader._composer=A
		A.anchors={}
	@property
	def parser(self):
		A=self
		if hasattr(A.loader,_B):A.loader.parser
		return A.loader._parser
	@property
	def resolver(self):
		A=self
		if hasattr(A.loader,_B):A.loader.resolver
		return A.loader._resolver
	def check_node(A):
		if A.parser.check_event(StreamStartEvent):A.parser.get_event()
		return not A.parser.check_event(StreamEndEvent)
	def get_node(A):
		if not A.parser.check_event(StreamEndEvent):return A.compose_document()
	def get_single_node(A):
		A.parser.get_event();B=_A
		if not A.parser.check_event(StreamEndEvent):B=A.compose_document()
		if not A.parser.check_event(StreamEndEvent):C=A.parser.get_event();raise ComposerError('expected a single document in the stream',B.start_mark,'but found another document',C.start_mark)
		A.parser.get_event();return B
	def compose_document(A):A.parser.get_event();B=A.compose_node(_A,_A);A.parser.get_event();A.anchors={};return B
	def compose_node(A,parent,index):
		if A.parser.check_event(AliasEvent):
			C=A.parser.get_event();D=C.anchor
			if D not in A.anchors:raise ComposerError(_A,_A,'found undefined alias %r'%utf8(D),C.start_mark)
			return A.anchors[D]
		C=A.parser.peek_event();B=C.anchor
		if B is not _A:
			if B in A.anchors:F='\nfound duplicate anchor {!r}\nfirst occurrence {}\nsecond occurrence {}'.format(B,A.anchors[B].start_mark,C.start_mark);warnings.warn(F,ReusedAnchorWarning)
		A.resolver.descend_resolver(parent,index)
		if A.parser.check_event(ScalarEvent):E=A.compose_scalar_node(B)
		elif A.parser.check_event(SequenceStartEvent):E=A.compose_sequence_node(B)
		elif A.parser.check_event(MappingStartEvent):E=A.compose_mapping_node(B)
		A.resolver.ascend_resolver();return E
	def compose_scalar_node(C,anchor):
		D=anchor;A=C.parser.get_event();B=A.tag
		if B is _A or B=='!':B=C.resolver.resolve(ScalarNode,A.value,A.implicit)
		E=ScalarNode(B,A.value,A.start_mark,A.end_mark,style=A.style,comment=A.comment,anchor=D)
		if D is not _A:C.anchors[D]=E
		return E
	def compose_sequence_node(B,anchor):
		F=anchor;C=B.parser.get_event();D=C.tag
		if D is _A or D=='!':D=B.resolver.resolve(SequenceNode,_A,C.implicit)
		A=SequenceNode(D,[],C.start_mark,_A,flow_style=C.flow_style,comment=C.comment,anchor=F)
		if F is not _A:B.anchors[F]=A
		G=0
		while not B.parser.check_event(SequenceEndEvent):A.value.append(B.compose_node(A,G));G+=1
		E=B.parser.get_event()
		if A.flow_style is True and E.comment is not _A:
			if A.comment is not _A:nprint('Warning: unexpected end_event commment in sequence node {}'.format(A.flow_style))
			A.comment=E.comment
		A.end_mark=E.end_mark;B.check_end_doc_comment(E,A);return A
	def compose_mapping_node(B,anchor):
		F=anchor;C=B.parser.get_event();D=C.tag
		if D is _A or D=='!':D=B.resolver.resolve(MappingNode,_A,C.implicit)
		A=MappingNode(D,[],C.start_mark,_A,flow_style=C.flow_style,comment=C.comment,anchor=F)
		if F is not _A:B.anchors[F]=A
		while not B.parser.check_event(MappingEndEvent):G=B.compose_node(A,_A);H=B.compose_node(A,G);A.value.append((G,H))
		E=B.parser.get_event()
		if A.flow_style is True and E.comment is not _A:A.comment=E.comment
		A.end_mark=E.end_mark;B.check_end_doc_comment(E,A);return A
	def check_end_doc_comment(C,end_event,node):
		B=node;A=end_event
		if A.comment and A.comment[1]:
			if B.comment is _A:B.comment=[_A,_A]
			assert not isinstance(B,ScalarEvent);B.comment.append(A.comment[1]);A.comment[1]=_A


import json
import os
import re
import warnings
from functools import wraps

from dynaconf.utils import extract_json_objects
from dynaconf.utils import multi_replace
from dynaconf.utils import recursively_evaluate_lazy_format
from dynaconf.utils.boxing import DynaBox
from dynaconf.vendor import toml

try:
    from jinja2 import Environment

    jinja_env = Environment()
    for p_method in ("abspath", "realpath", "relpath", "dirname", "basename"):
        jinja_env.filters[p_method] = getattr(os.path, p_method)
except ImportError:  # pragma: no cover
    jinja_env = None

true_values = ("t", "true", "enabled", "1", "on", "yes", "True")
false_values = ("f", "false", "disabled", "0", "off", "no", "False", "")


KV_PATTERN = re.compile(r"([a-zA-Z0-9 ]*=[a-zA-Z0-9\- :]*)")
"""matches `a=b, c=d, e=f` used on `VALUE='@merge foo=bar'` variables."""


class DynaconfParseError(Exception):
    """Error to raise when parsing @casts"""


class MetaValue:
    """A Marker to trigger specific actions on `set` and `object_merge`"""

    _meta_value = True

    def __init__(self, value, box_settings):
        self.box_settings = box_settings
        self.value = parse_conf_data(
            value, tomlfy=True, box_settings=box_settings
        )

    def __repr__(self):
        return f"{self.__class__.__name__}({self.value}) on {id(self)}"

    def unwrap(self):
        return self.value


class Reset(MetaValue):
    """Triggers an existing key to be reset to its value
    NOTE: DEPRECATED on v3.0.0
    """

    _dynaconf_reset = True

    def __init__(self, value, box_settings):
        self.box_settings = box_settings
        self.value = parse_conf_data(
            value, tomlfy=True, box_settings=self.box_settings
        )
        warnings.warn(f"{self.value} does not need `@reset` anymore.")


class Del(MetaValue):
    """Triggers an existing key to be deleted"""

    _dynaconf_del = True

    def unwrap(self):
        raise ValueError("Del object has no value")


class Merge(MetaValue):
    """Triggers an existing key to be merged"""

    _dynaconf_merge = True

    def __init__(self, value, box_settings, unique=False):
        self.box_settings = box_settings

        self.value = parse_conf_data(
            value, tomlfy=True, box_settings=box_settings
        )

        if isinstance(self.value, (int, float, bool)):
            # @merge 1, @merge 1.1, @merge False
            self.value = [self.value]
        elif isinstance(self.value, str):
            # @merge {"valid": "json"}
            json_object = list(
                extract_json_objects(
                    multi_replace(
                        self.value,
                        {
                            ": True": ": true",
                            ":True": ": true",
                            ": False": ": false",
                            ":False": ": false",
                            ": None": ": null",
                            ":None": ": null",
                        },
                    )
                )
            )
            if len(json_object) == 1:
                self.value = json_object[0]
            else:
                matches = KV_PATTERN.findall(self.value)
                # a=b, c=d
                if matches:
                    self.value = {
                        k.strip(): parse_conf_data(
                            v, tomlfy=True, box_settings=box_settings
                        )
                        for k, v in (
                            match.strip().split("=") for match in matches
                        )
                    }
                elif "," in self.value:
                    # @merge foo,bar
                    self.value = self.value.split(",")
                else:
                    # @merge foo
                    self.value = [self.value]

        self.unique = unique


class BaseFormatter:
    def __init__(self, function, token):
        self.function = function
        self.token = token

    def __call__(self, value, **context):
        return self.function(value, **context)

    def __str__(self):
        return str(self.token)


def _jinja_formatter(value, **context):
    if jinja_env is None:  # pragma: no cover
        raise ImportError(
            "jinja2 must be installed to enable '@jinja' settings in dynaconf"
        )
    return jinja_env.from_string(value).render(**context)


class Formatters:
    """Dynaconf builtin formatters"""

    python_formatter = BaseFormatter(str.format, "format")
    jinja_formatter = BaseFormatter(_jinja_formatter, "jinja")


class Lazy:
    """Holds data to format lazily."""

    _dynaconf_lazy_format = True

    def __init__(self, value, formatter=Formatters.python_formatter):
        self.value = value
        self.formatter = formatter

    @property
    def context(self):
        """Builds a context for formatting."""
        return {"env": os.environ, "this": self.settings}

    def __call__(self, settings):
        """LazyValue triggers format lazily."""
        self.settings = settings
        return self.formatter(self.value, **self.context)

    def __str__(self):
        """Gives string representation for the object."""
        return str(self.value)

    def __repr__(self):
        """Give the quoted str representation"""
        return f"'@{self.formatter} {self.value}'"

    def _dynaconf_encode(self):
        """Encodes this object values to be serializable to json"""
        return f"@{self.formatter} {self.value}"


def try_to_encode(value, callback=str):
    """Tries to encode a value by verifying existence of `_dynaconf_encode`"""
    try:
        return value._dynaconf_encode()
    except (AttributeError, TypeError):
        return callback(value)


def evaluate_lazy_format(f):
    """Marks a method on Settings instance to
    lazily evaluate LazyFormat objects upon access."""

    @wraps(f)
    def evaluate(settings, *args, **kwargs):
        value = f(settings, *args, **kwargs)
        return recursively_evaluate_lazy_format(value, settings)

    return evaluate


converters = {
    "@int": int,
    "@float": float,
    "@bool": lambda value: str(value).lower() in true_values,
    "@json": json.loads,
    "@format": lambda value: Lazy(value),
    "@jinja": lambda value: Lazy(value, formatter=Formatters.jinja_formatter),
    # Meta Values to trigger pre assignment actions
    "@reset": lambda value, box_settings: Reset(
        value, box_settings
    ),  # @reset is DEPRECATED on v3.0.0
    "@del": lambda value, box_settings: Del(value, box_settings),
    "@merge": lambda value, box_settings: Merge(value, box_settings),
    "@merge_unique": lambda value, box_settings: Merge(
        value, box_settings, unique=True
    ),
    # Special markers to be used as placeholders e.g: in prefilled forms
    # will always return None when evaluated
    "@note": lambda value: None,
    "@comment": lambda value: None,
    "@null": lambda value: None,
    "@none": lambda value: None,
}


def get_converter(converter_key, value, box_settings):
    converter = converters[converter_key]
    try:
        converted_value = converter(value, box_settings=box_settings)
    except TypeError:
        converted_value = converter(value)
    return converted_value


def parse_with_toml(data):
    """Uses TOML syntax to parse data"""
    try:
        return toml.loads(f"key={data}")["key"]
    except (toml.TomlDecodeError, KeyError):
        return data


def _parse_conf_data(data, tomlfy=False, box_settings=None):
    """
    @int @bool @float @json (for lists and dicts)
    strings does not need converters

    export DYNACONF_DEFAULT_THEME='material'
    export DYNACONF_DEBUG='@bool True'
    export DYNACONF_DEBUG_TOOLBAR_ENABLED='@bool False'
    export DYNACONF_PAGINATION_PER_PAGE='@int 20'
    export DYNACONF_MONGODB_SETTINGS='@json {"DB": "quokka_db"}'
    export DYNACONF_ALLOWED_EXTENSIONS='@json ["jpg", "png"]'
    """
    # not enforced to not break backwards compatibility with custom loaders
    box_settings = box_settings or {}

    cast_toggler = os.environ.get("AUTO_CAST_FOR_DYNACONF", "true").lower()
    castenabled = cast_toggler not in false_values

    if (
        castenabled
        and data
        and isinstance(data, str)
        and data.startswith(tuple(converters.keys()))
    ):
        parts = data.partition(" ")
        converter_key = parts[0]
        value = parts[-1]
        value = get_converter(converter_key, value, box_settings)
    else:
        value = parse_with_toml(data) if tomlfy else data

    if isinstance(value, dict):
        value = DynaBox(value, box_settings=box_settings)

    return value


def parse_conf_data(data, tomlfy=False, box_settings=None):

    # not enforced to not break backwards compatibility with custom loaders
    box_settings = box_settings or {}

    if isinstance(data, (tuple, list)):
        # recursively parse each sequence item
        return [
            parse_conf_data(item, tomlfy=tomlfy, box_settings=box_settings)
            for item in data
        ]
    elif isinstance(data, (dict, DynaBox)):
        # recursively parse inner dict items
        _parsed = {}
        for k, v in data.items():
            _parsed[k] = parse_conf_data(
                v, tomlfy=tomlfy, box_settings=box_settings
            )
        return _parsed
    else:
        # return parsed string value
        return _parse_conf_data(data, tomlfy=tomlfy, box_settings=box_settings)


def unparse_conf_data(value):
    if isinstance(value, bool):
        return f"@bool {value}"
    elif isinstance(value, int):
        return f"@int {value}"
    elif isinstance(value, float):
        return f"@float {value}"
    elif isinstance(value, (list, dict)):
        return f"@json {json.dumps(value)}"
    elif isinstance(value, Lazy):
        return try_to_encode(value)
    elif value is None:
        return "@none "
    else:
        return value



from __future__ import unicode_literals
_F=', line: '
_E='pre_done'
_D=False
_C='value'
_B='_comment'
_A=None
if _D:from typing import Text,Any,Dict,Optional,List;from .error import StreamMark
SHOWLINES=True
class Token:
	__slots__='start_mark','end_mark',_B
	def __init__(A,start_mark,end_mark):A.start_mark=start_mark;A.end_mark=end_mark
	def __repr__(A):
		C=[B for B in A.__slots__ if not B.endswith('_mark')];C.sort();B=', '.join(['%s=%r'%(B,getattr(A,B))for B in C])
		if SHOWLINES:
			try:B+=_F+str(A.start_mark.line)
			except:pass
		try:B+=', comment: '+str(A._comment)
		except:pass
		return '{}({})'.format(A.__class__.__name__,B)
	def add_post_comment(A,comment):
		if not hasattr(A,_B):A._comment=[_A,_A]
		A._comment[0]=comment
	def add_pre_comments(A,comments):
		if not hasattr(A,_B):A._comment=[_A,_A]
		assert A._comment[1]is _A;A._comment[1]=comments
	def get_comment(A):return getattr(A,_B,_A)
	@property
	def comment(self):return getattr(self,_B,_A)
	def move_comment(C,target,empty=_D):
		D=target;A=C.comment
		if A is _A:return
		if isinstance(D,(StreamEndToken,DocumentStartToken)):return
		delattr(C,_B);B=D.comment
		if not B:
			if empty:A=[A[0],A[1],_A,_A,A[0]]
			D._comment=A;return C
		if A[0]and B[0]or A[1]and B[1]:raise NotImplementedError('overlap in comment %r %r'%(A,B))
		if A[0]:B[0]=A[0]
		if A[1]:B[1]=A[1]
		return C
	def split_comment(B):
		A=B.comment
		if A is _A or A[0]is _A:return _A
		C=[A[0],_A]
		if A[1]is _A:delattr(B,_B)
		return C
class DirectiveToken(Token):
	__slots__='name',_C;id='<directive>'
	def __init__(A,name,value,start_mark,end_mark):Token.__init__(A,start_mark,end_mark);A.name=name;A.value=value
class DocumentStartToken(Token):__slots__=();id='<document start>'
class DocumentEndToken(Token):__slots__=();id='<document end>'
class StreamStartToken(Token):
	__slots__='encoding',;id='<stream start>'
	def __init__(A,start_mark=_A,end_mark=_A,encoding=_A):Token.__init__(A,start_mark,end_mark);A.encoding=encoding
class StreamEndToken(Token):__slots__=();id='<stream end>'
class BlockSequenceStartToken(Token):__slots__=();id='<block sequence start>'
class BlockMappingStartToken(Token):__slots__=();id='<block mapping start>'
class BlockEndToken(Token):__slots__=();id='<block end>'
class FlowSequenceStartToken(Token):__slots__=();id='['
class FlowMappingStartToken(Token):__slots__=();id='{'
class FlowSequenceEndToken(Token):__slots__=();id=']'
class FlowMappingEndToken(Token):__slots__=();id='}'
class KeyToken(Token):__slots__=();id='?'
class ValueToken(Token):__slots__=();id=':'
class BlockEntryToken(Token):__slots__=();id='-'
class FlowEntryToken(Token):__slots__=();id=','
class AliasToken(Token):
	__slots__=_C,;id='<alias>'
	def __init__(A,value,start_mark,end_mark):Token.__init__(A,start_mark,end_mark);A.value=value
class AnchorToken(Token):
	__slots__=_C,;id='<anchor>'
	def __init__(A,value,start_mark,end_mark):Token.__init__(A,start_mark,end_mark);A.value=value
class TagToken(Token):
	__slots__=_C,;id='<tag>'
	def __init__(A,value,start_mark,end_mark):Token.__init__(A,start_mark,end_mark);A.value=value
class ScalarToken(Token):
	__slots__=_C,'plain','style';id='<scalar>'
	def __init__(A,value,plain,start_mark,end_mark,style=_A):Token.__init__(A,start_mark,end_mark);A.value=value;A.plain=plain;A.style=style
class CommentToken(Token):
	__slots__=_C,_E;id='<comment>'
	def __init__(A,value,start_mark,end_mark):Token.__init__(A,start_mark,end_mark);A.value=value
	def reset(A):
		if hasattr(A,_E):delattr(A,_E)
	def __repr__(A):
		B='{!r}'.format(A.value)
		if SHOWLINES:
			try:B+=_F+str(A.start_mark.line);B+=', col: '+str(A.start_mark.column)
			except:pass
		return 'CommentToken({})'.format(B)
	def __eq__(A,other):
		B=other
		if A.start_mark!=B.start_mark:return _D
		if A.end_mark!=B.end_mark:return _D
		if A.value!=B.value:return _D
		return True
	def __ne__(A,other):return not A.__eq__(other)


from __future__ import absolute_import
_F='expected <block end>, but found %r'
_E='typ'
_D='!'
_C=True
_B=False
_A=None
from .error import MarkedYAMLError
from .tokens import *
from .events import *
from .scanner import Scanner,RoundTripScanner,ScannerError
from .compat import utf8,nprint,nprintf
if _B:from typing import Any,Dict,Optional,List
__all__=['Parser','RoundTripParser','ParserError']
class ParserError(MarkedYAMLError):0
class Parser:
	DEFAULT_TAGS={_D:_D,'!!':'tag:yaml.org,2002:'}
	def __init__(self,loader):
		self.loader=loader
		if self.loader is not _A and getattr(self.loader,'_parser',_A)is _A:self.loader._parser=self
		self.reset_parser()
	def reset_parser(self):self.current_event=_A;self.tag_handles={};self.states=[];self.marks=[];self.state=self.parse_stream_start
	def dispose(self):self.reset_parser()
	@property
	def scanner(self):
		if hasattr(self.loader,_E):return self.loader.scanner
		return self.loader._scanner
	@property
	def resolver(self):
		if hasattr(self.loader,_E):return self.loader.resolver
		return self.loader._resolver
	def check_event(self,*choices):
		if self.current_event is _A:
			if self.state:self.current_event=self.state()
		if self.current_event is not _A:
			if not choices:return _C
			for choice in choices:
				if isinstance(self.current_event,choice):return _C
		return _B
	def peek_event(self):
		if self.current_event is _A:
			if self.state:self.current_event=self.state()
		return self.current_event
	def get_event(self):
		if self.current_event is _A:
			if self.state:self.current_event=self.state()
		value=self.current_event;self.current_event=_A;return value
	def parse_stream_start(self):token=self.scanner.get_token();token.move_comment(self.scanner.peek_token());event=StreamStartEvent(token.start_mark,token.end_mark,encoding=token.encoding);self.state=self.parse_implicit_document_start;return event
	def parse_implicit_document_start(self):
		if not self.scanner.check_token(DirectiveToken,DocumentStartToken,StreamEndToken):self.tag_handles=self.DEFAULT_TAGS;token=self.scanner.peek_token();start_mark=end_mark=token.start_mark;event=DocumentStartEvent(start_mark,end_mark,explicit=_B);self.states.append(self.parse_document_end);self.state=self.parse_block_node;return event
		else:return self.parse_document_start()
	def parse_document_start(self):
		while self.scanner.check_token(DocumentEndToken):self.scanner.get_token()
		if not self.scanner.check_token(StreamEndToken):
			token=self.scanner.peek_token();start_mark=token.start_mark;version,tags=self.process_directives()
			if not self.scanner.check_token(DocumentStartToken):raise ParserError(_A,_A,"expected '<document start>', but found %r"%self.scanner.peek_token().id,self.scanner.peek_token().start_mark)
			token=self.scanner.get_token();end_mark=token.end_mark;event=DocumentStartEvent(start_mark,end_mark,explicit=_C,version=version,tags=tags);self.states.append(self.parse_document_end);self.state=self.parse_document_content
		else:token=self.scanner.get_token();event=StreamEndEvent(token.start_mark,token.end_mark,comment=token.comment);assert not self.states;assert not self.marks;self.state=_A
		return event
	def parse_document_end(self):
		token=self.scanner.peek_token();start_mark=end_mark=token.start_mark;explicit=_B
		if self.scanner.check_token(DocumentEndToken):token=self.scanner.get_token();end_mark=token.end_mark;explicit=_C
		event=DocumentEndEvent(start_mark,end_mark,explicit=explicit)
		if self.resolver.processing_version==(1,1):self.state=self.parse_document_start
		else:self.state=self.parse_implicit_document_start
		return event
	def parse_document_content(self):
		if self.scanner.check_token(DirectiveToken,DocumentStartToken,DocumentEndToken,StreamEndToken):event=self.process_empty_scalar(self.scanner.peek_token().start_mark);self.state=self.states.pop();return event
		else:return self.parse_block_node()
	def process_directives(self):
		yaml_version=_A;self.tag_handles={}
		while self.scanner.check_token(DirectiveToken):
			token=self.scanner.get_token()
			if token.name=='YAML':
				if yaml_version is not _A:raise ParserError(_A,_A,'found duplicate YAML directive',token.start_mark)
				major,minor=token.value
				if major!=1:raise ParserError(_A,_A,'found incompatible YAML document (version 1.* is required)',token.start_mark)
				yaml_version=token.value
			elif token.name=='TAG':
				handle,prefix=token.value
				if handle in self.tag_handles:raise ParserError(_A,_A,'duplicate tag handle %r'%utf8(handle),token.start_mark)
				self.tag_handles[handle]=prefix
		if bool(self.tag_handles):value=yaml_version,self.tag_handles.copy()
		else:value=yaml_version,_A
		if self.loader is not _A and hasattr(self.loader,'tags'):
			self.loader.version=yaml_version
			if self.loader.tags is _A:self.loader.tags={}
			for k in self.tag_handles:self.loader.tags[k]=self.tag_handles[k]
		for key in self.DEFAULT_TAGS:
			if key not in self.tag_handles:self.tag_handles[key]=self.DEFAULT_TAGS[key]
		return value
	def parse_block_node(self):return self.parse_node(block=_C)
	def parse_flow_node(self):return self.parse_node()
	def parse_block_node_or_indentless_sequence(self):return self.parse_node(block=_C,indentless_sequence=_C)
	def transform_tag(self,handle,suffix):return self.tag_handles[handle]+suffix
	def parse_node(self,block=_B,indentless_sequence=_B):
		if self.scanner.check_token(AliasToken):token=self.scanner.get_token();event=AliasEvent(token.value,token.start_mark,token.end_mark);self.state=self.states.pop();return event
		anchor=_A;tag=_A;start_mark=end_mark=tag_mark=_A
		if self.scanner.check_token(AnchorToken):
			token=self.scanner.get_token();start_mark=token.start_mark;end_mark=token.end_mark;anchor=token.value
			if self.scanner.check_token(TagToken):token=self.scanner.get_token();tag_mark=token.start_mark;end_mark=token.end_mark;tag=token.value
		elif self.scanner.check_token(TagToken):
			token=self.scanner.get_token();start_mark=tag_mark=token.start_mark;end_mark=token.end_mark;tag=token.value
			if self.scanner.check_token(AnchorToken):token=self.scanner.get_token();start_mark=tag_mark=token.start_mark;end_mark=token.end_mark;anchor=token.value
		if tag is not _A:
			handle,suffix=tag
			if handle is not _A:
				if handle not in self.tag_handles:raise ParserError('while parsing a node',start_mark,'found undefined tag handle %r'%utf8(handle),tag_mark)
				tag=self.transform_tag(handle,suffix)
			else:tag=suffix
		if start_mark is _A:start_mark=end_mark=self.scanner.peek_token().start_mark
		event=_A;implicit=tag is _A or tag==_D
		if indentless_sequence and self.scanner.check_token(BlockEntryToken):
			comment=_A;pt=self.scanner.peek_token()
			if pt.comment and pt.comment[0]:comment=[pt.comment[0],[]];pt.comment[0]=_A
			end_mark=self.scanner.peek_token().end_mark;event=SequenceStartEvent(anchor,tag,implicit,start_mark,end_mark,flow_style=_B,comment=comment);self.state=self.parse_indentless_sequence_entry;return event
		if self.scanner.check_token(ScalarToken):
			token=self.scanner.get_token();end_mark=token.end_mark
			if token.plain and tag is _A or tag==_D:implicit=_C,_B
			elif tag is _A:implicit=_B,_C
			else:implicit=_B,_B
			event=ScalarEvent(anchor,tag,implicit,token.value,start_mark,end_mark,style=token.style,comment=token.comment);self.state=self.states.pop()
		elif self.scanner.check_token(FlowSequenceStartToken):pt=self.scanner.peek_token();end_mark=pt.end_mark;event=SequenceStartEvent(anchor,tag,implicit,start_mark,end_mark,flow_style=_C,comment=pt.comment);self.state=self.parse_flow_sequence_first_entry
		elif self.scanner.check_token(FlowMappingStartToken):pt=self.scanner.peek_token();end_mark=pt.end_mark;event=MappingStartEvent(anchor,tag,implicit,start_mark,end_mark,flow_style=_C,comment=pt.comment);self.state=self.parse_flow_mapping_first_key
		elif block and self.scanner.check_token(BlockSequenceStartToken):
			end_mark=self.scanner.peek_token().start_mark;pt=self.scanner.peek_token();comment=pt.comment
			if comment is _A or comment[1]is _A:comment=pt.split_comment()
			event=SequenceStartEvent(anchor,tag,implicit,start_mark,end_mark,flow_style=_B,comment=comment);self.state=self.parse_block_sequence_first_entry
		elif block and self.scanner.check_token(BlockMappingStartToken):end_mark=self.scanner.peek_token().start_mark;comment=self.scanner.peek_token().comment;event=MappingStartEvent(anchor,tag,implicit,start_mark,end_mark,flow_style=_B,comment=comment);self.state=self.parse_block_mapping_first_key
		elif anchor is not _A or tag is not _A:event=ScalarEvent(anchor,tag,(implicit,_B),'',start_mark,end_mark);self.state=self.states.pop()
		else:
			if block:node='block'
			else:node='flow'
			token=self.scanner.peek_token();raise ParserError('while parsing a %s node'%node,start_mark,'expected the node content, but found %r'%token.id,token.start_mark)
		return event
	def parse_block_sequence_first_entry(self):token=self.scanner.get_token();self.marks.append(token.start_mark);return self.parse_block_sequence_entry()
	def parse_block_sequence_entry(self):
		if self.scanner.check_token(BlockEntryToken):
			token=self.scanner.get_token();token.move_comment(self.scanner.peek_token())
			if not self.scanner.check_token(BlockEntryToken,BlockEndToken):self.states.append(self.parse_block_sequence_entry);return self.parse_block_node()
			else:self.state=self.parse_block_sequence_entry;return self.process_empty_scalar(token.end_mark)
		if not self.scanner.check_token(BlockEndToken):token=self.scanner.peek_token();raise ParserError('while parsing a block collection',self.marks[-1],_F%token.id,token.start_mark)
		token=self.scanner.get_token();event=SequenceEndEvent(token.start_mark,token.end_mark,comment=token.comment);self.state=self.states.pop();self.marks.pop();return event
	def parse_indentless_sequence_entry(self):
		if self.scanner.check_token(BlockEntryToken):
			token=self.scanner.get_token();token.move_comment(self.scanner.peek_token())
			if not self.scanner.check_token(BlockEntryToken,KeyToken,ValueToken,BlockEndToken):self.states.append(self.parse_indentless_sequence_entry);return self.parse_block_node()
			else:self.state=self.parse_indentless_sequence_entry;return self.process_empty_scalar(token.end_mark)
		token=self.scanner.peek_token();event=SequenceEndEvent(token.start_mark,token.start_mark,comment=token.comment);self.state=self.states.pop();return event
	def parse_block_mapping_first_key(self):token=self.scanner.get_token();self.marks.append(token.start_mark);return self.parse_block_mapping_key()
	def parse_block_mapping_key(self):
		if self.scanner.check_token(KeyToken):
			token=self.scanner.get_token();token.move_comment(self.scanner.peek_token())
			if not self.scanner.check_token(KeyToken,ValueToken,BlockEndToken):self.states.append(self.parse_block_mapping_value);return self.parse_block_node_or_indentless_sequence()
			else:self.state=self.parse_block_mapping_value;return self.process_empty_scalar(token.end_mark)
		if self.resolver.processing_version>(1,1)and self.scanner.check_token(ValueToken):self.state=self.parse_block_mapping_value;return self.process_empty_scalar(self.scanner.peek_token().start_mark)
		if not self.scanner.check_token(BlockEndToken):token=self.scanner.peek_token();raise ParserError('while parsing a block mapping',self.marks[-1],_F%token.id,token.start_mark)
		token=self.scanner.get_token();token.move_comment(self.scanner.peek_token());event=MappingEndEvent(token.start_mark,token.end_mark,comment=token.comment);self.state=self.states.pop();self.marks.pop();return event
	def parse_block_mapping_value(self):
		if self.scanner.check_token(ValueToken):
			token=self.scanner.get_token()
			if self.scanner.check_token(ValueToken):token.move_comment(self.scanner.peek_token())
			elif not self.scanner.check_token(KeyToken):token.move_comment(self.scanner.peek_token(),empty=_C)
			if not self.scanner.check_token(KeyToken,ValueToken,BlockEndToken):self.states.append(self.parse_block_mapping_key);return self.parse_block_node_or_indentless_sequence()
			else:
				self.state=self.parse_block_mapping_key;comment=token.comment
				if comment is _A:
					token=self.scanner.peek_token();comment=token.comment
					if comment:token._comment=[_A,comment[1]];comment=[comment[0],_A]
				return self.process_empty_scalar(token.end_mark,comment=comment)
		else:self.state=self.parse_block_mapping_key;token=self.scanner.peek_token();return self.process_empty_scalar(token.start_mark)
	def parse_flow_sequence_first_entry(self):token=self.scanner.get_token();self.marks.append(token.start_mark);return self.parse_flow_sequence_entry(first=_C)
	def parse_flow_sequence_entry(self,first=_B):
		if not self.scanner.check_token(FlowSequenceEndToken):
			if not first:
				if self.scanner.check_token(FlowEntryToken):self.scanner.get_token()
				else:token=self.scanner.peek_token();raise ParserError('while parsing a flow sequence',self.marks[-1],"expected ',' or ']', but got %r"%token.id,token.start_mark)
			if self.scanner.check_token(KeyToken):token=self.scanner.peek_token();event=MappingStartEvent(_A,_A,_C,token.start_mark,token.end_mark,flow_style=_C);self.state=self.parse_flow_sequence_entry_mapping_key;return event
			elif not self.scanner.check_token(FlowSequenceEndToken):self.states.append(self.parse_flow_sequence_entry);return self.parse_flow_node()
		token=self.scanner.get_token();event=SequenceEndEvent(token.start_mark,token.end_mark,comment=token.comment);self.state=self.states.pop();self.marks.pop();return event
	def parse_flow_sequence_entry_mapping_key(self):
		token=self.scanner.get_token()
		if not self.scanner.check_token(ValueToken,FlowEntryToken,FlowSequenceEndToken):self.states.append(self.parse_flow_sequence_entry_mapping_value);return self.parse_flow_node()
		else:self.state=self.parse_flow_sequence_entry_mapping_value;return self.process_empty_scalar(token.end_mark)
	def parse_flow_sequence_entry_mapping_value(self):
		if self.scanner.check_token(ValueToken):
			token=self.scanner.get_token()
			if not self.scanner.check_token(FlowEntryToken,FlowSequenceEndToken):self.states.append(self.parse_flow_sequence_entry_mapping_end);return self.parse_flow_node()
			else:self.state=self.parse_flow_sequence_entry_mapping_end;return self.process_empty_scalar(token.end_mark)
		else:self.state=self.parse_flow_sequence_entry_mapping_end;token=self.scanner.peek_token();return self.process_empty_scalar(token.start_mark)
	def parse_flow_sequence_entry_mapping_end(self):self.state=self.parse_flow_sequence_entry;token=self.scanner.peek_token();return MappingEndEvent(token.start_mark,token.start_mark)
	def parse_flow_mapping_first_key(self):token=self.scanner.get_token();self.marks.append(token.start_mark);return self.parse_flow_mapping_key(first=_C)
	def parse_flow_mapping_key(self,first=_B):
		if not self.scanner.check_token(FlowMappingEndToken):
			if not first:
				if self.scanner.check_token(FlowEntryToken):self.scanner.get_token()
				else:token=self.scanner.peek_token();raise ParserError('while parsing a flow mapping',self.marks[-1],"expected ',' or '}', but got %r"%token.id,token.start_mark)
			if self.scanner.check_token(KeyToken):
				token=self.scanner.get_token()
				if not self.scanner.check_token(ValueToken,FlowEntryToken,FlowMappingEndToken):self.states.append(self.parse_flow_mapping_value);return self.parse_flow_node()
				else:self.state=self.parse_flow_mapping_value;return self.process_empty_scalar(token.end_mark)
			elif self.resolver.processing_version>(1,1)and self.scanner.check_token(ValueToken):self.state=self.parse_flow_mapping_value;return self.process_empty_scalar(self.scanner.peek_token().end_mark)
			elif not self.scanner.check_token(FlowMappingEndToken):self.states.append(self.parse_flow_mapping_empty_value);return self.parse_flow_node()
		token=self.scanner.get_token();event=MappingEndEvent(token.start_mark,token.end_mark,comment=token.comment);self.state=self.states.pop();self.marks.pop();return event
	def parse_flow_mapping_value(self):
		if self.scanner.check_token(ValueToken):
			token=self.scanner.get_token()
			if not self.scanner.check_token(FlowEntryToken,FlowMappingEndToken):self.states.append(self.parse_flow_mapping_key);return self.parse_flow_node()
			else:self.state=self.parse_flow_mapping_key;return self.process_empty_scalar(token.end_mark)
		else:self.state=self.parse_flow_mapping_key;token=self.scanner.peek_token();return self.process_empty_scalar(token.start_mark)
	def parse_flow_mapping_empty_value(self):self.state=self.parse_flow_mapping_key;return self.process_empty_scalar(self.scanner.peek_token().start_mark)
	def process_empty_scalar(self,mark,comment=_A):return ScalarEvent(_A,_A,(_C,_B),'',mark,mark,comment=comment)
class RoundTripParser(Parser):
	def transform_tag(self,handle,suffix):
		if handle=='!!'and suffix in('null','bool','int','float','binary','timestamp','omap','pairs','set','str','seq','map'):return Parser.transform_tag(self,handle,suffix)
		return handle+suffix


_H='explicit'
_G='style'
_F='flow_style'
_E='value'
_D='anchor'
_C='implicit'
_B='tag'
_A=None
if False:from typing import Any,Dict,Optional,List
def CommentCheck():0
class Event:
	__slots__='start_mark','end_mark','comment'
	def __init__(A,start_mark=_A,end_mark=_A,comment=CommentCheck):
		B=comment;A.start_mark=start_mark;A.end_mark=end_mark
		if B is CommentCheck:B=_A
		A.comment=B
	def __repr__(A):
		C=[B for B in[_D,_B,_C,_E,_F,_G]if hasattr(A,B)];B=', '.join(['%s=%r'%(B,getattr(A,B))for B in C])
		if A.comment not in[_A,CommentCheck]:B+=', comment={!r}'.format(A.comment)
		return'%s(%s)'%(A.__class__.__name__,B)
class NodeEvent(Event):
	__slots__=_D,
	def __init__(A,anchor,start_mark=_A,end_mark=_A,comment=_A):Event.__init__(A,start_mark,end_mark,comment);A.anchor=anchor
class CollectionStartEvent(NodeEvent):
	__slots__=_B,_C,_F,'nr_items'
	def __init__(A,anchor,tag,implicit,start_mark=_A,end_mark=_A,flow_style=_A,comment=_A,nr_items=_A):NodeEvent.__init__(A,anchor,start_mark,end_mark,comment);A.tag=tag;A.implicit=implicit;A.flow_style=flow_style;A.nr_items=nr_items
class CollectionEndEvent(Event):__slots__=()
class StreamStartEvent(Event):
	__slots__='encoding',
	def __init__(A,start_mark=_A,end_mark=_A,encoding=_A,comment=_A):Event.__init__(A,start_mark,end_mark,comment);A.encoding=encoding
class StreamEndEvent(Event):__slots__=()
class DocumentStartEvent(Event):
	__slots__=_H,'version','tags'
	def __init__(A,start_mark=_A,end_mark=_A,explicit=_A,version=_A,tags=_A,comment=_A):Event.__init__(A,start_mark,end_mark,comment);A.explicit=explicit;A.version=version;A.tags=tags
class DocumentEndEvent(Event):
	__slots__=_H,
	def __init__(A,start_mark=_A,end_mark=_A,explicit=_A,comment=_A):Event.__init__(A,start_mark,end_mark,comment);A.explicit=explicit
class AliasEvent(NodeEvent):__slots__=()
class ScalarEvent(NodeEvent):
	__slots__=_B,_C,_E,_G
	def __init__(A,anchor,tag,implicit,value,start_mark=_A,end_mark=_A,style=_A,comment=_A):NodeEvent.__init__(A,anchor,start_mark,end_mark,comment);A.tag=tag;A.implicit=implicit;A.value=value;A.style=style
class SequenceStartEvent(CollectionStartEvent):__slots__=()
class SequenceEndEvent(CollectionEndEvent):__slots__=()
class MappingStartEvent(CollectionStartEvent):__slots__=()
class MappingEndEvent(CollectionEndEvent):__slots__=()


import io
import os
import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages


def read(*names, **kwargs):
    """Read a file."""
    content = ""
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8"),
    ) as open_file:
        content = open_file.read().strip()
    return content


setup(
    name="dynaconf",
    version=read("dynaconf", "VERSION"),
    url="https://github.com/rochacbruno/dynaconf",
    license="MIT",
    author="Bruno Rocha",
    author_email="rochacbruno@gmail.com",
    description="The dynamic configurator for your Python Project",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(
        exclude=[
            "tests",
            "tests.*",
            "example",
            "example.*",
            "docs",
            "legacy_docs",
            "legacy_docs.*",
            "docs.*",
            "build",
            "build.*",
            "dynaconf.vendor_src",
            "dynaconf/vendor_src",
            "dynaconf.vendor_src.*",
            "dynaconf/vendor_src/*",
        ]
    ),
    include_package_data=True,
    zip_safe=False,
    platforms="any",
    install_requires=["typing; python_version<'3.5'"],
    tests_require=[
        "pytest",
        "pytest-cov",
        "pytest-xdist",
        "flake8",
        "pep8-naming",
        "flake8-debugger",
        "flake8-print",
        "flake8-todo",
        "radon",
        "flask>=0.12",
        "python-dotenv",
        "toml",
        "codecov",
    ],
    extras_require={
        "redis": ["redis"],
        "vault": ["hvac"],
        "yaml": ["ruamel.yaml"],
        "toml": ["toml"],
        "ini": ["configobj"],
        "configobj": ["configobj"],
        "all": ["redis", "ruamel.yaml", "configobj", "hvac"],
    },
    entry_points={"console_scripts": ["dynaconf=dynaconf.cli:main"]},
    setup_requires=["setuptools>=38.6.0"]
    if sys.version_info >= (3, 6, 0)
    else [],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Framework :: Flask",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



_J='is_eager'
_I='prompt'
_H='expose_value'
_G='callback'
_F='is_flag'
_E='cls'
_D=False
_C=True
_B='help'
_A=None
import inspect,sys
from functools import update_wrapper
from .core import Argument
from .core import Command
from .core import Group
from .core import Option
from .globals import get_current_context
from .utils import echo
def pass_context(f):
	'Marks a callback as wanting to receive the current context\n    object as first argument.\n    '
	def A(*A,**B):return f(get_current_context(),*A,**B)
	return update_wrapper(A,f)
def pass_obj(f):
	'Similar to :func:`pass_context`, but only pass the object on the\n    context onwards (:attr:`Context.obj`).  This is useful if that object\n    represents the state of a nested system.\n    '
	def A(*A,**B):return f(get_current_context().obj,*A,**B)
	return update_wrapper(A,f)
def make_pass_decorator(object_type,ensure=_D):
	"Given an object type this creates a decorator that will work\n    similar to :func:`pass_obj` but instead of passing the object of the\n    current context, it will find the innermost context of type\n    :func:`object_type`.\n\n    This generates a decorator that works roughly like this::\n\n        from functools import update_wrapper\n\n        def decorator(f):\n            @pass_context\n            def new_func(ctx, *args, **kwargs):\n                obj = ctx.find_object(object_type)\n                return ctx.invoke(f, obj, *args, **kwargs)\n            return update_wrapper(new_func, f)\n        return decorator\n\n    :param object_type: the type of the object to pass.\n    :param ensure: if set to `True`, a new object will be created and\n                   remembered on the context if it's not there yet.\n    ";A=object_type
	def B(f):
		def B(*D,**E):
			B=get_current_context()
			if ensure:C=B.ensure_object(A)
			else:C=B.find_object(A)
			if C is _A:raise RuntimeError(f"Managed to invoke callback without a context object of type {A.__name__!r} existing.")
			return B.invoke(f,C,*D,**E)
		return update_wrapper(B,f)
	return B
def _make_command(f,name,attrs,cls):
	A=attrs
	if isinstance(f,Command):raise TypeError('Attempted to convert a callback into a command twice.')
	try:B=f.__click_params__;B.reverse();del f.__click_params__
	except AttributeError:B=[]
	help=A.get(_B)
	if help is _A:
		help=inspect.getdoc(f)
		if isinstance(help,bytes):help=help.decode('utf-8')
	else:help=inspect.cleandoc(help)
	A[_B]=help;return cls(name=name or f.__name__.lower().replace('_','-'),callback=f,params=B,**A)
def command(name=_A,cls=_A,**C):
	'Creates a new :class:`Command` and uses the decorated function as\n    callback.  This will also automatically attach all decorated\n    :func:`option`\\s and :func:`argument`\\s as parameters to the command.\n\n    The name of the command defaults to the name of the function with\n    underscores replaced by dashes.  If you want to change that, you can\n    pass the intended name as the first argument.\n\n    All keyword arguments are forwarded to the underlying command class.\n\n    Once decorated the function turns into a :class:`Command` instance\n    that can be invoked as a command line utility or be attached to a\n    command :class:`Group`.\n\n    :param name: the name of the command.  This defaults to the function\n                 name with underscores replaced by dashes.\n    :param cls: the command class to instantiate.  This defaults to\n                :class:`Command`.\n    ';A=cls
	if A is _A:A=Command
	def B(f):B=_make_command(f,name,C,A);B.__doc__=f.__doc__;return B
	return B
def group(name=_A,**A):'Creates a new :class:`Group` with a function as callback.  This\n    works otherwise the same as :func:`command` just that the `cls`\n    parameter is set to :class:`Group`.\n    ';A.setdefault(_E,Group);return command(name,**A)
def _param_memo(f,param):
	A=param
	if isinstance(f,Command):f.params.append(A)
	else:
		if not hasattr(f,'__click_params__'):f.__click_params__=[]
		f.__click_params__.append(A)
def argument(*B,**A):
	'Attaches an argument to the command.  All positional arguments are\n    passed as parameter declarations to :class:`Argument`; all keyword\n    arguments are forwarded unchanged (except ``cls``).\n    This is equivalent to creating an :class:`Argument` instance manually\n    and attaching it to the :attr:`Command.params` list.\n\n    :param cls: the argument class to instantiate.  This defaults to\n                :class:`Argument`.\n    '
	def C(f):C=A.pop(_E,Argument);_param_memo(f,C(B,**A));return f
	return C
def option(*B,**C):
	'Attaches an option to the command.  All positional arguments are\n    passed as parameter declarations to :class:`Option`; all keyword\n    arguments are forwarded unchanged (except ``cls``).\n    This is equivalent to creating an :class:`Option` instance manually\n    and attaching it to the :attr:`Command.params` list.\n\n    :param cls: the option class to instantiate.  This defaults to\n                :class:`Option`.\n    '
	def A(f):
		A=C.copy()
		if _B in A:A[_B]=inspect.cleandoc(A[_B])
		D=A.pop(_E,Option);_param_memo(f,D(B,**A));return f
	return A
def confirmation_option(*B,**A):
	"Shortcut for confirmation prompts that can be ignored by passing\n    ``--yes`` as parameter.\n\n    This is equivalent to decorating a function with :func:`option` with\n    the following parameters::\n\n        def callback(ctx, param, value):\n            if not value:\n                ctx.abort()\n\n        @click.command()\n        @click.option('--yes', is_flag=True, callback=callback,\n                      expose_value=False, prompt='Do you want to continue?')\n        def dropdb():\n            pass\n    "
	def C(f):
		def C(ctx,param,value):
			if not value:ctx.abort()
		A.setdefault(_F,_C);A.setdefault(_G,C);A.setdefault(_H,_D);A.setdefault(_I,'Do you want to continue?');A.setdefault(_B,'Confirm the action without prompting.');return option(*B or('--yes',),**A)(f)
	return C
def password_option(*B,**A):
	"Shortcut for password prompts.\n\n    This is equivalent to decorating a function with :func:`option` with\n    the following parameters::\n\n        @click.command()\n        @click.option('--password', prompt=True, confirmation_prompt=True,\n                      hide_input=True)\n        def changeadmin(password):\n            pass\n    "
	def C(f):A.setdefault(_I,_C);A.setdefault('confirmation_prompt',_C);A.setdefault('hide_input',_C);return option(*B or('--password',),**A)(f)
	return C
def version_option(version=_A,*B,**A):
	"Adds a ``--version`` option which immediately ends the program\n    printing out the version number.  This is implemented as an eager\n    option that prints the version and exits the program in the callback.\n\n    :param version: the version number to show.  If not provided Click\n                    attempts an auto discovery via setuptools.\n    :param prog_name: the name of the program (defaults to autodetection)\n    :param message: custom message to show instead of the default\n                    (``'%(prog)s, version %(version)s'``)\n    :param others: everything else is forwarded to :func:`option`.\n    ";D=version
	if D is _A:
		if hasattr(sys,'_getframe'):E=sys._getframe(1).f_globals.get('__name__')
		else:E=''
	def C(f):
		G=A.pop('prog_name',_A);H=A.pop('message','%(prog)s, version %(version)s')
		def C(ctx,param,value):
			A=ctx
			if not value or A.resilient_parsing:return
			C=G
			if C is _A:C=A.find_root().info_name
			B=D
			if B is _A:
				try:import pkg_resources as I
				except ImportError:pass
				else:
					for F in I.working_set:
						J=F.get_entry_map().get('console_scripts')or{}
						for K in J.values():
							if K.module_name==E:B=F.version;break
				if B is _A:raise RuntimeError('Could not determine version')
			echo(H%{'prog':C,'version':B},color=A.color);A.exit()
		A.setdefault(_F,_C);A.setdefault(_H,_D);A.setdefault(_J,_C);A.setdefault(_B,'Show the version and exit.');A[_G]=C;return option(*B or('--version',),**A)(f)
	return C
def help_option(*B,**A):
	'Adds a ``--help`` option which immediately ends the program\n    printing out the help page.  This is usually unnecessary to add as\n    this is added by default to all commands unless suppressed.\n\n    Like :func:`version_option`, this is implemented as eager option that\n    prints in the callback and exits.\n\n    All arguments are forwarded to :func:`option`.\n    '
	def C(f):
		def C(ctx,param,value):
			A=ctx
			if value and not A.resilient_parsing:echo(A.get_help(),color=A.color);A.exit()
		A.setdefault(_F,_C);A.setdefault(_H,_D);A.setdefault(_B,'Show this message and exit.');A.setdefault(_J,_C);A[_G]=C;return option(*B or('--help',),**A)(f)
	return C


import io
from pathlib import Path

from dynaconf import default_settings
from dynaconf.constants import TOML_EXTENSIONS
from dynaconf.loaders.base import BaseLoader
from dynaconf.utils import object_merge
from dynaconf.vendor import toml


def load(obj, env=None, silent=True, key=None, filename=None):
    """
    Reads and loads in to "obj" a single key or all keys from source file.

    :param obj: the settings instance
    :param env: settings current env default='development'
    :param silent: if errors should raise
    :param key: if defined load a single key, else load all in env
    :param filename: Optional custom filename to load
    :return: None
    """

    loader = BaseLoader(
        obj=obj,
        env=env,
        identifier="toml",
        extensions=TOML_EXTENSIONS,
        file_reader=toml.load,
        string_reader=toml.loads,
    )
    loader.load(filename=filename, key=key, silent=silent)


def write(settings_path, settings_data, merge=True):
    """Write data to a settings file.

    :param settings_path: the filepath
    :param settings_data: a dictionary with data
    :param merge: boolean if existing file should be merged with new data
    """
    settings_path = Path(settings_path)
    if settings_path.exists() and merge:  # pragma: no cover
        with io.open(
            str(settings_path), encoding=default_settings.ENCODING_FOR_DYNACONF
        ) as open_file:
            object_merge(toml.load(open_file), settings_data)

    with io.open(
        str(settings_path),
        "w",
        encoding=default_settings.ENCODING_FOR_DYNACONF,
    ) as open_file:
        toml.dump(encode_nulls(settings_data), open_file)


def encode_nulls(data):
    """TOML does not support `None` so this function transforms to '@none '."""
    if data is None:
        return "@none "
    if isinstance(data, dict):
        return {key: encode_nulls(value) for key, value in data.items()}
    elif isinstance(data, (list, tuple)):
        return [encode_nulls(item) for item in data]
    return data



import warnings

try:
    from flask.config import Config

    flask_installed = True
except ImportError:  # pragma: no cover
    flask_installed = False
    Config = object


import dynaconf
from importlib import import_module


class FlaskDynaconf(object):
    """The arguments are.
    app = The created app
    dynaconf_args = Extra args to be passed to Dynaconf (validator for example)

    All other values are stored as config vars specially::

        ENVVAR_PREFIX_FOR_DYNACONF = env prefix for your envvars to be loaded
                            example:
                                if you set to `MYSITE` then
                                export MYSITE_SQL_PORT='@int 5445'

                            with that exported to env you access using:
                                app.config.SQL_PORT
                                app.config.get('SQL_PORT')
                                app.config.get('sql_port')
                                # get is case insensitive
                                app.config['SQL_PORT']

                            Dynaconf uses `@int, @bool, @float, @json` to cast
                            env vars

        SETTINGS_FILE_FOR_DYNACONF = The name of the module or file to use as
                                    default to load settings. If nothing is
                                    passed it will be `settings.*` or value
                                    found in `ENVVAR_FOR_DYNACONF`
                                    Dynaconf supports
                                    .py, .yml, .toml, ini, json

    ATTENTION: Take a look at `settings.yml` and `.secrets.yml` to know the
            required settings format.

    Settings load order in Dynaconf:

    - Load all defaults and Flask defaults
    - Load all passed variables when applying FlaskDynaconf
    - Update with data in settings files
    - Update with data in environment vars `ENVVAR_FOR_DYNACONF_`


    TOML files are very useful to have `envd` settings, lets say,
    `production` and `development`.

    You can also achieve the same using multiple `.py` files naming as
    `settings.py`, `production_settings.py` and `development_settings.py`
    (see examples/validator)

    Example::

        app = Flask(__name__)
        FlaskDynaconf(
            app,
            ENV='MYSITE',
            SETTINGS_FILE='settings.yml',
            EXTRA_VALUE='You can add aditional config vars here'
        )

    Take a look at examples/flask in Dynaconf repository

    """

    def __init__(
        self,
        app=None,
        instance_relative_config=False,
        dynaconf_instance=None,
        extensions_list=False,
        **kwargs,
    ):
        """kwargs holds initial dynaconf configuration"""
        if not flask_installed:  # pragma: no cover
            raise RuntimeError(
                "To use this extension Flask must be installed "
                "install it with: pip install flask"
            )
        self.kwargs = kwargs

        kwargs.setdefault("ENVVAR_PREFIX", "FLASK")
        env_prefix = f"{kwargs['ENVVAR_PREFIX']}_ENV"  # FLASK_ENV
        kwargs.setdefault("ENV_SWITCHER", env_prefix)
        kwargs.setdefault("ENVIRONMENTS", True)
        kwargs.setdefault("load_dotenv", True)
        kwargs.setdefault(
            "default_settings_paths", dynaconf.DEFAULT_SETTINGS_FILES
        )

        self.dynaconf_instance = dynaconf_instance
        self.instance_relative_config = instance_relative_config
        self.extensions_list = extensions_list
        if app:
            self.init_app(app, **kwargs)

    def init_app(self, app, **kwargs):
        """kwargs holds initial dynaconf configuration"""
        self.kwargs.update(kwargs)
        self.settings = self.dynaconf_instance or dynaconf.LazySettings(
            **self.kwargs
        )
        dynaconf.settings = self.settings  # rebind customized settings
        app.config = self.make_config(app)
        app.dynaconf = self.settings

        if self.extensions_list:
            if not isinstance(self.extensions_list, str):
                self.extensions_list = "EXTENSIONS"
            app.config.load_extensions(self.extensions_list)

    def make_config(self, app):
        root_path = app.root_path
        if self.instance_relative_config:  # pragma: no cover
            root_path = app.instance_path
        if self.dynaconf_instance:
            self.settings.update(self.kwargs)
        return DynaconfConfig(
            root_path=root_path,
            defaults=app.config,
            _settings=self.settings,
            _app=app,
        )


class DynaconfConfig(Config):
    """
    Settings load order in Dynaconf:

    - Load all defaults and Flask defaults
    - Load all passed variables when applying FlaskDynaconf
    - Update with data in settings files
    - Update with data in environmente vars `ENV_FOR_DYNACONF_`
    """

    def __init__(self, _settings, _app, *args, **kwargs):
        """perform the initial load"""
        super(DynaconfConfig, self).__init__(*args, **kwargs)
        Config.update(self, _settings.store)
        self._settings = _settings
        self._app = _app

    def __getitem__(self, key):
        """
        Flask templates always expects a None when key is not found in config
        """
        return self.get(key)

    def __setitem__(self, key, value):
        """
        Allows app.config['key'] = 'foo'
        """
        return self._settings.__setitem__(key, value)

    def __getattr__(self, name):
        """
        First try to get value from dynaconf then from Flask
        """
        try:
            return getattr(self._settings, name)
        except AttributeError:
            return self[name]

    def __call__(self, name, *args, **kwargs):
        return self.get(name, *args, **kwargs)

    def get(self, key, default=None):
        """Gets config from dynaconf variables
        if variables does not exists in dynaconf try getting from
        `app.config` to support runtime settings."""
        return self._settings.get(key, Config.get(self, key, default))

    def load_extensions(self, key="EXTENSIONS", app=None):
        """Loads flask extensions dynamically."""
        app = app or self._app
        extensions = app.config.get(key)
        if not extensions:
            warnings.warn(
                f"Settings is missing {key} to load Flask Extensions",
                RuntimeWarning,
            )
            return

        for extension in app.config[key]:
            # Split data in form `extension.path:factory_function`
            module_name, factory = extension.split(":")
            # Dynamically import extension module.
            ext = import_module(module_name)
            # Invoke factory passing app.
            getattr(ext, factory)(app)



from __future__ import print_function,absolute_import,division
_AD='expected the empty value, but found %r'
_AC='cannot find module %r (%s)'
_AB='expected non-empty name appended to the tag'
_AA='tag:yaml.org,2002:map'
_A9='tag:yaml.org,2002:seq'
_A8='tag:yaml.org,2002:set'
_A7='tag:yaml.org,2002:pairs'
_A6='tag:yaml.org,2002:omap'
_A5='tag:yaml.org,2002:timestamp'
_A4='tag:yaml.org,2002:binary'
_A3='tag:yaml.org,2002:float'
_A2='tag:yaml.org,2002:int'
_A1='tag:yaml.org,2002:bool'
_A0='tag:yaml.org,2002:null'
_z='could not determine a constructor for the tag %r'
_y='second'
_x='minute'
_w='day'
_v='month'
_u='year'
_t='failed to construct timestamp from "{}"'
_s='decodebytes'
_r='failed to convert base64 data into ascii: %s'
_q='.nan'
_p='.inf'
_o='expected a mapping or list of mappings for merging, but found %s'
_n='expected a mapping for merging, but found %s'
_m='                        Duplicate keys will become an error in future releases, and are errors\n                        by default when using the new API.\n                        '
_l='\n                        To suppress this check see:\n                           http://yaml.readthedocs.io/en/latest/api.html#duplicate-keys\n                        '
_k='tag:yaml.org,2002:merge'
_j='                    Duplicate keys will become an error in future releases, and are errors\n                    by default when using the new API.\n                    '
_i='\n                    To suppress this check see:\n                        http://yaml.readthedocs.io/en/latest/api.html#duplicate-keys\n                    '
_h='expected a sequence node, but found %s'
_g='expected a scalar node, but found %s'
_f='typ'
_e='while constructing a Python module'
_d='expected a single mapping item, but found %d items'
_c='expected a mapping of length 1, but found %s'
_b='expected a sequence, but found %s'
_a='failed to decode base64 data: %s'
_Z='tag:yaml.org,2002:value'
_Y='found duplicate key "{}"'
_X='found unhashable key'
_W='found unacceptable key (%s)'
_V='__setstate__'
_U='tz_hour'
_T='hour'
_S='ascii'
_R='tag:yaml.org,2002:str'
_Q='utf-8'
_P='expected a mapping node, but found %s'
_O='tz_minute'
_N='e'
_M='+-'
_L='while constructing an ordered map'
_K='tz_sign'
_J='-'
_I='fraction'
_H='.'
_G=':'
_F='0'
_E='while constructing a mapping'
_D='_'
_C=True
_B=False
_A=None
import datetime,base64,binascii,re,sys,types,warnings
from .error import MarkedYAMLError,MarkedYAMLFutureWarning,MantissaNoDotYAML1_1Warning
from .nodes import *
from .nodes import SequenceNode,MappingNode,ScalarNode
from .compat import utf8,builtins_module,to_str,PY2,PY3,text_type,nprint,nprintf,version_tnf
from .compat import ordereddict,Hashable,MutableSequence
from .compat import MutableMapping
from .comments import *
from .comments import CommentedMap,CommentedOrderedMap,CommentedSet,CommentedKeySeq,CommentedSeq,TaggedScalar,CommentedKeyMap
from .scalarstring import SingleQuotedScalarString,DoubleQuotedScalarString,LiteralScalarString,FoldedScalarString,PlainScalarString,ScalarString
from .scalarint import ScalarInt,BinaryInt,OctalInt,HexInt,HexCapsInt
from .scalarfloat import ScalarFloat
from .scalarbool import ScalarBoolean
from .timestamp import TimeStamp
from .util import RegExp
if _B:from typing import Any,Dict,List,Set,Generator,Union,Optional
__all__=['BaseConstructor','SafeConstructor','Constructor','ConstructorError','RoundTripConstructor']
class ConstructorError(MarkedYAMLError):0
class DuplicateKeyFutureWarning(MarkedYAMLFutureWarning):0
class DuplicateKeyError(MarkedYAMLFutureWarning):0
class BaseConstructor:
	yaml_constructors={};yaml_multi_constructors={}
	def __init__(self,preserve_quotes=_A,loader=_A):
		self.loader=loader
		if self.loader is not _A and getattr(self.loader,'_constructor',_A)is _A:self.loader._constructor=self
		self.loader=loader;self.yaml_base_dict_type=dict;self.yaml_base_list_type=list;self.constructed_objects={};self.recursive_objects={};self.state_generators=[];self.deep_construct=_B;self._preserve_quotes=preserve_quotes;self.allow_duplicate_keys=version_tnf((0,15,1),(0,16))
	@property
	def composer(self):
		if hasattr(self.loader,_f):return self.loader.composer
		try:return self.loader._composer
		except AttributeError:sys.stdout.write('slt {}\n'.format(type(self)));sys.stdout.write('slc {}\n'.format(self.loader._composer));sys.stdout.write('{}\n'.format(dir(self)));raise
	@property
	def resolver(self):
		if hasattr(self.loader,_f):return self.loader.resolver
		return self.loader._resolver
	def check_data(self):return self.composer.check_node()
	def get_data(self):
		if self.composer.check_node():return self.construct_document(self.composer.get_node())
	def get_single_data(self):
		node=self.composer.get_single_node()
		if node is not _A:return self.construct_document(node)
		return _A
	def construct_document(self,node):
		data=self.construct_object(node)
		while bool(self.state_generators):
			state_generators=self.state_generators;self.state_generators=[]
			for generator in state_generators:
				for _dummy in generator:0
		self.constructed_objects={};self.recursive_objects={};self.deep_construct=_B;return data
	def construct_object(self,node,deep=_B):
		if node in self.constructed_objects:return self.constructed_objects[node]
		if deep:old_deep=self.deep_construct;self.deep_construct=_C
		if node in self.recursive_objects:return self.recursive_objects[node]
		self.recursive_objects[node]=_A;data=self.construct_non_recursive_object(node);self.constructed_objects[node]=data;del self.recursive_objects[node]
		if deep:self.deep_construct=old_deep
		return data
	def construct_non_recursive_object(self,node,tag=_A):
		constructor=_A;tag_suffix=_A
		if tag is _A:tag=node.tag
		if tag in self.yaml_constructors:constructor=self.yaml_constructors[tag]
		else:
			for tag_prefix in self.yaml_multi_constructors:
				if tag.startswith(tag_prefix):tag_suffix=tag[len(tag_prefix):];constructor=self.yaml_multi_constructors[tag_prefix];break
			else:
				if _A in self.yaml_multi_constructors:tag_suffix=tag;constructor=self.yaml_multi_constructors[_A]
				elif _A in self.yaml_constructors:constructor=self.yaml_constructors[_A]
				elif isinstance(node,ScalarNode):constructor=self.__class__.construct_scalar
				elif isinstance(node,SequenceNode):constructor=self.__class__.construct_sequence
				elif isinstance(node,MappingNode):constructor=self.__class__.construct_mapping
		if tag_suffix is _A:data=constructor(self,node)
		else:data=constructor(self,tag_suffix,node)
		if isinstance(data,types.GeneratorType):
			generator=data;data=next(generator)
			if self.deep_construct:
				for _dummy in generator:0
			else:self.state_generators.append(generator)
		return data
	def construct_scalar(self,node):
		if not isinstance(node,ScalarNode):raise ConstructorError(_A,_A,_g%node.id,node.start_mark)
		return node.value
	def construct_sequence(self,node,deep=_B):
		if not isinstance(node,SequenceNode):raise ConstructorError(_A,_A,_h%node.id,node.start_mark)
		return[self.construct_object(child,deep=deep)for child in node.value]
	def construct_mapping(self,node,deep=_B):
		if not isinstance(node,MappingNode):raise ConstructorError(_A,_A,_P%node.id,node.start_mark)
		total_mapping=self.yaml_base_dict_type()
		if getattr(node,'merge',_A)is not _A:todo=[(node.merge,_B),(node.value,_B)]
		else:todo=[(node.value,_C)]
		for (values,check) in todo:
			mapping=self.yaml_base_dict_type()
			for (key_node,value_node) in values:
				key=self.construct_object(key_node,deep=_C)
				if not isinstance(key,Hashable):
					if isinstance(key,list):key=tuple(key)
				if PY2:
					try:hash(key)
					except TypeError as exc:raise ConstructorError(_E,node.start_mark,_W%exc,key_node.start_mark)
				elif not isinstance(key,Hashable):raise ConstructorError(_E,node.start_mark,_X,key_node.start_mark)
				value=self.construct_object(value_node,deep=deep)
				if check:
					if self.check_mapping_key(node,key_node,mapping,key,value):mapping[key]=value
				else:mapping[key]=value
			total_mapping.update(mapping)
		return total_mapping
	def check_mapping_key(self,node,key_node,mapping,key,value):
		if key in mapping:
			if not self.allow_duplicate_keys:
				mk=mapping.get(key)
				if PY2:
					if isinstance(key,unicode):key=key.encode(_Q)
					if isinstance(value,unicode):value=value.encode(_Q)
					if isinstance(mk,unicode):mk=mk.encode(_Q)
				args=[_E,node.start_mark,'found duplicate key "{}" with value "{}" (original value: "{}")'.format(key,value,mk),key_node.start_mark,_i,_j]
				if self.allow_duplicate_keys is _A:warnings.warn(DuplicateKeyFutureWarning(*args))
				else:raise DuplicateKeyError(*args)
			return _B
		return _C
	def check_set_key(self,node,key_node,setting,key):
		if key in setting:
			if not self.allow_duplicate_keys:
				if PY2:
					if isinstance(key,unicode):key=key.encode(_Q)
				args=['while constructing a set',node.start_mark,_Y.format(key),key_node.start_mark,_i,_j]
				if self.allow_duplicate_keys is _A:warnings.warn(DuplicateKeyFutureWarning(*args))
				else:raise DuplicateKeyError(*args)
	def construct_pairs(self,node,deep=_B):
		if not isinstance(node,MappingNode):raise ConstructorError(_A,_A,_P%node.id,node.start_mark)
		pairs=[]
		for (key_node,value_node) in node.value:key=self.construct_object(key_node,deep=deep);value=self.construct_object(value_node,deep=deep);pairs.append((key,value))
		return pairs
	@classmethod
	def add_constructor(cls,tag,constructor):
		if'yaml_constructors'not in cls.__dict__:cls.yaml_constructors=cls.yaml_constructors.copy()
		cls.yaml_constructors[tag]=constructor
	@classmethod
	def add_multi_constructor(cls,tag_prefix,multi_constructor):
		if'yaml_multi_constructors'not in cls.__dict__:cls.yaml_multi_constructors=cls.yaml_multi_constructors.copy()
		cls.yaml_multi_constructors[tag_prefix]=multi_constructor
class SafeConstructor(BaseConstructor):
	def construct_scalar(self,node):
		if isinstance(node,MappingNode):
			for (key_node,value_node) in node.value:
				if key_node.tag==_Z:return self.construct_scalar(value_node)
		return BaseConstructor.construct_scalar(self,node)
	def flatten_mapping(self,node):
		merge=[];index=0
		while index<len(node.value):
			key_node,value_node=node.value[index]
			if key_node.tag==_k:
				if merge:
					if self.allow_duplicate_keys:del node.value[index];index+=1;continue
					args=[_E,node.start_mark,_Y.format(key_node.value),key_node.start_mark,_l,_m]
					if self.allow_duplicate_keys is _A:warnings.warn(DuplicateKeyFutureWarning(*args))
					else:raise DuplicateKeyError(*args)
				del node.value[index]
				if isinstance(value_node,MappingNode):self.flatten_mapping(value_node);merge.extend(value_node.value)
				elif isinstance(value_node,SequenceNode):
					submerge=[]
					for subnode in value_node.value:
						if not isinstance(subnode,MappingNode):raise ConstructorError(_E,node.start_mark,_n%subnode.id,subnode.start_mark)
						self.flatten_mapping(subnode);submerge.append(subnode.value)
					submerge.reverse()
					for value in submerge:merge.extend(value)
				else:raise ConstructorError(_E,node.start_mark,_o%value_node.id,value_node.start_mark)
			elif key_node.tag==_Z:key_node.tag=_R;index+=1
			else:index+=1
		if bool(merge):node.merge=merge;node.value=merge+node.value
	def construct_mapping(self,node,deep=_B):
		if isinstance(node,MappingNode):self.flatten_mapping(node)
		return BaseConstructor.construct_mapping(self,node,deep=deep)
	def construct_yaml_null(self,node):self.construct_scalar(node);return _A
	bool_values={'yes':_C,'no':_B,'y':_C,'n':_B,'true':_C,'false':_B,'on':_C,'off':_B}
	def construct_yaml_bool(self,node):value=self.construct_scalar(node);return self.bool_values[value.lower()]
	def construct_yaml_int(self,node):
		value_s=to_str(self.construct_scalar(node));value_s=value_s.replace(_D,'');sign=+1
		if value_s[0]==_J:sign=-1
		if value_s[0]in _M:value_s=value_s[1:]
		if value_s==_F:return 0
		elif value_s.startswith('0b'):return sign*int(value_s[2:],2)
		elif value_s.startswith('0x'):return sign*int(value_s[2:],16)
		elif value_s.startswith('0o'):return sign*int(value_s[2:],8)
		elif self.resolver.processing_version==(1,1)and value_s[0]==_F:return sign*int(value_s,8)
		elif self.resolver.processing_version==(1,1)and _G in value_s:
			digits=[int(part)for part in value_s.split(_G)];digits.reverse();base=1;value=0
			for digit in digits:value+=digit*base;base*=60
			return sign*value
		else:return sign*int(value_s)
	inf_value=1e+300
	while inf_value!=inf_value*inf_value:inf_value*=inf_value
	nan_value=-inf_value/inf_value
	def construct_yaml_float(self,node):
		value_so=to_str(self.construct_scalar(node));value_s=value_so.replace(_D,'').lower();sign=+1
		if value_s[0]==_J:sign=-1
		if value_s[0]in _M:value_s=value_s[1:]
		if value_s==_p:return sign*self.inf_value
		elif value_s==_q:return self.nan_value
		elif self.resolver.processing_version!=(1,2)and _G in value_s:
			digits=[float(part)for part in value_s.split(_G)];digits.reverse();base=1;value=0.0
			for digit in digits:value+=digit*base;base*=60
			return sign*value
		else:
			if self.resolver.processing_version!=(1,2)and _N in value_s:
				mantissa,exponent=value_s.split(_N)
				if _H not in mantissa:warnings.warn(MantissaNoDotYAML1_1Warning(node,value_so))
			return sign*float(value_s)
	if PY3:
		def construct_yaml_binary(self,node):
			try:value=self.construct_scalar(node).encode(_S)
			except UnicodeEncodeError as exc:raise ConstructorError(_A,_A,_r%exc,node.start_mark)
			try:
				if hasattr(base64,_s):return base64.decodebytes(value)
				else:return base64.decodestring(value)
			except binascii.Error as exc:raise ConstructorError(_A,_A,_a%exc,node.start_mark)
	else:
		def construct_yaml_binary(self,node):
			value=self.construct_scalar(node)
			try:return to_str(value).decode('base64')
			except (binascii.Error,UnicodeEncodeError)as exc:raise ConstructorError(_A,_A,_a%exc,node.start_mark)
	timestamp_regexp=RegExp('^(?P<year>[0-9][0-9][0-9][0-9])\n          -(?P<month>[0-9][0-9]?)\n          -(?P<day>[0-9][0-9]?)\n          (?:((?P<t>[Tt])|[ \\t]+)   # explictly not retaining extra spaces\n          (?P<hour>[0-9][0-9]?)\n          :(?P<minute>[0-9][0-9])\n          :(?P<second>[0-9][0-9])\n          (?:\\.(?P<fraction>[0-9]*))?\n          (?:[ \\t]*(?P<tz>Z|(?P<tz_sign>[-+])(?P<tz_hour>[0-9][0-9]?)\n          (?::(?P<tz_minute>[0-9][0-9]))?))?)?$',re.X)
	def construct_yaml_timestamp(self,node,values=_A):
		if values is _A:
			try:match=self.timestamp_regexp.match(node.value)
			except TypeError:match=_A
			if match is _A:raise ConstructorError(_A,_A,_t.format(node.value),node.start_mark)
			values=match.groupdict()
		year=int(values[_u]);month=int(values[_v]);day=int(values[_w])
		if not values[_T]:return datetime.date(year,month,day)
		hour=int(values[_T]);minute=int(values[_x]);second=int(values[_y]);fraction=0
		if values[_I]:
			fraction_s=values[_I][:6]
			while len(fraction_s)<6:fraction_s+=_F
			fraction=int(fraction_s)
			if len(values[_I])>6 and int(values[_I][6])>4:fraction+=1
		delta=_A
		if values[_K]:
			tz_hour=int(values[_U]);minutes=values[_O];tz_minute=int(minutes)if minutes else 0;delta=datetime.timedelta(hours=tz_hour,minutes=tz_minute)
			if values[_K]==_J:delta=-delta
		data=datetime.datetime(year,month,day,hour,minute,second,fraction)
		if delta:data-=delta
		return data
	def construct_yaml_omap(self,node):
		omap=ordereddict();yield omap
		if not isinstance(node,SequenceNode):raise ConstructorError(_L,node.start_mark,_b%node.id,node.start_mark)
		for subnode in node.value:
			if not isinstance(subnode,MappingNode):raise ConstructorError(_L,node.start_mark,_c%subnode.id,subnode.start_mark)
			if len(subnode.value)!=1:raise ConstructorError(_L,node.start_mark,_d%len(subnode.value),subnode.start_mark)
			key_node,value_node=subnode.value[0];key=self.construct_object(key_node);assert key not in omap;value=self.construct_object(value_node);omap[key]=value
	def construct_yaml_pairs(self,node):
		A='while constructing pairs';pairs=[];yield pairs
		if not isinstance(node,SequenceNode):raise ConstructorError(A,node.start_mark,_b%node.id,node.start_mark)
		for subnode in node.value:
			if not isinstance(subnode,MappingNode):raise ConstructorError(A,node.start_mark,_c%subnode.id,subnode.start_mark)
			if len(subnode.value)!=1:raise ConstructorError(A,node.start_mark,_d%len(subnode.value),subnode.start_mark)
			key_node,value_node=subnode.value[0];key=self.construct_object(key_node);value=self.construct_object(value_node);pairs.append((key,value))
	def construct_yaml_set(self,node):data=set();yield data;value=self.construct_mapping(node);data.update(value)
	def construct_yaml_str(self,node):
		value=self.construct_scalar(node)
		if PY3:return value
		try:return value.encode(_S)
		except UnicodeEncodeError:return value
	def construct_yaml_seq(self,node):data=self.yaml_base_list_type();yield data;data.extend(self.construct_sequence(node))
	def construct_yaml_map(self,node):data=self.yaml_base_dict_type();yield data;value=self.construct_mapping(node);data.update(value)
	def construct_yaml_object(self,node,cls):
		data=cls.__new__(cls);yield data
		if hasattr(data,_V):state=self.construct_mapping(node,deep=_C);data.__setstate__(state)
		else:state=self.construct_mapping(node);data.__dict__.update(state)
	def construct_undefined(self,node):raise ConstructorError(_A,_A,_z%utf8(node.tag),node.start_mark)
SafeConstructor.add_constructor(_A0,SafeConstructor.construct_yaml_null)
SafeConstructor.add_constructor(_A1,SafeConstructor.construct_yaml_bool)
SafeConstructor.add_constructor(_A2,SafeConstructor.construct_yaml_int)
SafeConstructor.add_constructor(_A3,SafeConstructor.construct_yaml_float)
SafeConstructor.add_constructor(_A4,SafeConstructor.construct_yaml_binary)
SafeConstructor.add_constructor(_A5,SafeConstructor.construct_yaml_timestamp)
SafeConstructor.add_constructor(_A6,SafeConstructor.construct_yaml_omap)
SafeConstructor.add_constructor(_A7,SafeConstructor.construct_yaml_pairs)
SafeConstructor.add_constructor(_A8,SafeConstructor.construct_yaml_set)
SafeConstructor.add_constructor(_R,SafeConstructor.construct_yaml_str)
SafeConstructor.add_constructor(_A9,SafeConstructor.construct_yaml_seq)
SafeConstructor.add_constructor(_AA,SafeConstructor.construct_yaml_map)
SafeConstructor.add_constructor(_A,SafeConstructor.construct_undefined)
if PY2:
	class classobj:0
class Constructor(SafeConstructor):
	def construct_python_str(self,node):return utf8(self.construct_scalar(node))
	def construct_python_unicode(self,node):return self.construct_scalar(node)
	if PY3:
		def construct_python_bytes(self,node):
			try:value=self.construct_scalar(node).encode(_S)
			except UnicodeEncodeError as exc:raise ConstructorError(_A,_A,_r%exc,node.start_mark)
			try:
				if hasattr(base64,_s):return base64.decodebytes(value)
				else:return base64.decodestring(value)
			except binascii.Error as exc:raise ConstructorError(_A,_A,_a%exc,node.start_mark)
	def construct_python_long(self,node):
		val=self.construct_yaml_int(node)
		if PY3:return val
		return int(val)
	def construct_python_complex(self,node):return complex(self.construct_scalar(node))
	def construct_python_tuple(self,node):return tuple(self.construct_sequence(node))
	def find_python_module(self,name,mark):
		if not name:raise ConstructorError(_e,mark,_AB,mark)
		try:__import__(name)
		except ImportError as exc:raise ConstructorError(_e,mark,_AC%(utf8(name),exc),mark)
		return sys.modules[name]
	def find_python_name(self,name,mark):
		A='while constructing a Python object'
		if not name:raise ConstructorError(A,mark,_AB,mark)
		if _H in name:
			lname=name.split(_H);lmodule_name=lname;lobject_name=[]
			while len(lmodule_name)>1:
				lobject_name.insert(0,lmodule_name.pop());module_name=_H.join(lmodule_name)
				try:__import__(module_name);break
				except ImportError:continue
		else:module_name=builtins_module;lobject_name=[name]
		try:__import__(module_name)
		except ImportError as exc:raise ConstructorError(A,mark,_AC%(utf8(module_name),exc),mark)
		module=sys.modules[module_name];object_name=_H.join(lobject_name);obj=module
		while lobject_name:
			if not hasattr(obj,lobject_name[0]):raise ConstructorError(A,mark,'cannot find %r in the module %r'%(utf8(object_name),module.__name__),mark)
			obj=getattr(obj,lobject_name.pop(0))
		return obj
	def construct_python_name(self,suffix,node):
		value=self.construct_scalar(node)
		if value:raise ConstructorError('while constructing a Python name',node.start_mark,_AD%utf8(value),node.start_mark)
		return self.find_python_name(suffix,node.start_mark)
	def construct_python_module(self,suffix,node):
		value=self.construct_scalar(node)
		if value:raise ConstructorError(_e,node.start_mark,_AD%utf8(value),node.start_mark)
		return self.find_python_module(suffix,node.start_mark)
	def make_python_instance(self,suffix,node,args=_A,kwds=_A,newobj=_B):
		if not args:args=[]
		if not kwds:kwds={}
		cls=self.find_python_name(suffix,node.start_mark)
		if PY3:
			if newobj and isinstance(cls,type):return cls.__new__(cls,*args,**kwds)
			else:return cls(*args,**kwds)
		elif newobj and isinstance(cls,type(classobj))and not args and not kwds:instance=classobj();instance.__class__=cls;return instance
		elif newobj and isinstance(cls,type):return cls.__new__(cls,*args,**kwds)
		else:return cls(*args,**kwds)
	def set_python_instance_state(self,instance,state):
		if hasattr(instance,_V):instance.__setstate__(state)
		else:
			slotstate={}
			if isinstance(state,tuple)and len(state)==2:state,slotstate=state
			if hasattr(instance,'__dict__'):instance.__dict__.update(state)
			elif state:slotstate.update(state)
			for (key,value) in slotstate.items():setattr(instance,key,value)
	def construct_python_object(self,suffix,node):instance=self.make_python_instance(suffix,node,newobj=_C);self.recursive_objects[node]=instance;yield instance;deep=hasattr(instance,_V);state=self.construct_mapping(node,deep=deep);self.set_python_instance_state(instance,state)
	def construct_python_object_apply(self,suffix,node,newobj=_B):
		if isinstance(node,SequenceNode):args=self.construct_sequence(node,deep=_C);kwds={};state={};listitems=[];dictitems={}
		else:value=self.construct_mapping(node,deep=_C);args=value.get('args',[]);kwds=value.get('kwds',{});state=value.get('state',{});listitems=value.get('listitems',[]);dictitems=value.get('dictitems',{})
		instance=self.make_python_instance(suffix,node,args,kwds,newobj)
		if bool(state):self.set_python_instance_state(instance,state)
		if bool(listitems):instance.extend(listitems)
		if bool(dictitems):
			for key in dictitems:instance[key]=dictitems[key]
		return instance
	def construct_python_object_new(self,suffix,node):return self.construct_python_object_apply(suffix,node,newobj=_C)
Constructor.add_constructor('tag:yaml.org,2002:python/none',Constructor.construct_yaml_null)
Constructor.add_constructor('tag:yaml.org,2002:python/bool',Constructor.construct_yaml_bool)
Constructor.add_constructor('tag:yaml.org,2002:python/str',Constructor.construct_python_str)
Constructor.add_constructor('tag:yaml.org,2002:python/unicode',Constructor.construct_python_unicode)
if PY3:Constructor.add_constructor('tag:yaml.org,2002:python/bytes',Constructor.construct_python_bytes)
Constructor.add_constructor('tag:yaml.org,2002:python/int',Constructor.construct_yaml_int)
Constructor.add_constructor('tag:yaml.org,2002:python/long',Constructor.construct_python_long)
Constructor.add_constructor('tag:yaml.org,2002:python/float',Constructor.construct_yaml_float)
Constructor.add_constructor('tag:yaml.org,2002:python/complex',Constructor.construct_python_complex)
Constructor.add_constructor('tag:yaml.org,2002:python/list',Constructor.construct_yaml_seq)
Constructor.add_constructor('tag:yaml.org,2002:python/tuple',Constructor.construct_python_tuple)
Constructor.add_constructor('tag:yaml.org,2002:python/dict',Constructor.construct_yaml_map)
Constructor.add_multi_constructor('tag:yaml.org,2002:python/name:',Constructor.construct_python_name)
Constructor.add_multi_constructor('tag:yaml.org,2002:python/module:',Constructor.construct_python_module)
Constructor.add_multi_constructor('tag:yaml.org,2002:python/object:',Constructor.construct_python_object)
Constructor.add_multi_constructor('tag:yaml.org,2002:python/object/apply:',Constructor.construct_python_object_apply)
Constructor.add_multi_constructor('tag:yaml.org,2002:python/object/new:',Constructor.construct_python_object_new)
class RoundTripConstructor(SafeConstructor):
	def construct_scalar(self,node):
		A='\x07'
		if not isinstance(node,ScalarNode):raise ConstructorError(_A,_A,_g%node.id,node.start_mark)
		if node.style=='|'and isinstance(node.value,text_type):
			lss=LiteralScalarString(node.value,anchor=node.anchor)
			if node.comment and node.comment[1]:lss.comment=node.comment[1][0]
			return lss
		if node.style=='>'and isinstance(node.value,text_type):
			fold_positions=[];idx=-1
			while _C:
				idx=node.value.find(A,idx+1)
				if idx<0:break
				fold_positions.append(idx-len(fold_positions))
			fss=FoldedScalarString(node.value.replace(A,''),anchor=node.anchor)
			if node.comment and node.comment[1]:fss.comment=node.comment[1][0]
			if fold_positions:fss.fold_pos=fold_positions
			return fss
		elif bool(self._preserve_quotes)and isinstance(node.value,text_type):
			if node.style=="'":return SingleQuotedScalarString(node.value,anchor=node.anchor)
			if node.style=='"':return DoubleQuotedScalarString(node.value,anchor=node.anchor)
		if node.anchor:return PlainScalarString(node.value,anchor=node.anchor)
		return node.value
	def construct_yaml_int(self,node):
		width=_A;value_su=to_str(self.construct_scalar(node))
		try:sx=value_su.rstrip(_D);underscore=[len(sx)-sx.rindex(_D)-1,_B,_B]
		except ValueError:underscore=_A
		except IndexError:underscore=_A
		value_s=value_su.replace(_D,'');sign=+1
		if value_s[0]==_J:sign=-1
		if value_s[0]in _M:value_s=value_s[1:]
		if value_s==_F:return 0
		elif value_s.startswith('0b'):
			if self.resolver.processing_version>(1,1)and value_s[2]==_F:width=len(value_s[2:])
			if underscore is not _A:underscore[1]=value_su[2]==_D;underscore[2]=len(value_su[2:])>1 and value_su[-1]==_D
			return BinaryInt(sign*int(value_s[2:],2),width=width,underscore=underscore,anchor=node.anchor)
		elif value_s.startswith('0x'):
			if self.resolver.processing_version>(1,1)and value_s[2]==_F:width=len(value_s[2:])
			hex_fun=HexInt
			for ch in value_s[2:]:
				if ch in'ABCDEF':hex_fun=HexCapsInt;break
				if ch in'abcdef':break
			if underscore is not _A:underscore[1]=value_su[2]==_D;underscore[2]=len(value_su[2:])>1 and value_su[-1]==_D
			return hex_fun(sign*int(value_s[2:],16),width=width,underscore=underscore,anchor=node.anchor)
		elif value_s.startswith('0o'):
			if self.resolver.processing_version>(1,1)and value_s[2]==_F:width=len(value_s[2:])
			if underscore is not _A:underscore[1]=value_su[2]==_D;underscore[2]=len(value_su[2:])>1 and value_su[-1]==_D
			return OctalInt(sign*int(value_s[2:],8),width=width,underscore=underscore,anchor=node.anchor)
		elif self.resolver.processing_version!=(1,2)and value_s[0]==_F:return sign*int(value_s,8)
		elif self.resolver.processing_version!=(1,2)and _G in value_s:
			digits=[int(part)for part in value_s.split(_G)];digits.reverse();base=1;value=0
			for digit in digits:value+=digit*base;base*=60
			return sign*value
		elif self.resolver.processing_version>(1,1)and value_s[0]==_F:
			if underscore is not _A:underscore[2]=len(value_su)>1 and value_su[-1]==_D
			return ScalarInt(sign*int(value_s),width=len(value_s),underscore=underscore)
		elif underscore:underscore[2]=len(value_su)>1 and value_su[-1]==_D;return ScalarInt(sign*int(value_s),width=_A,underscore=underscore,anchor=node.anchor)
		elif node.anchor:return ScalarInt(sign*int(value_s),width=_A,anchor=node.anchor)
		else:return sign*int(value_s)
	def construct_yaml_float(self,node):
		A='E'
		def leading_zeros(v):
			lead0=0;idx=0
			while idx<len(v)and v[idx]in'0.':
				if v[idx]==_F:lead0+=1
				idx+=1
			return lead0
		m_sign=_B;value_so=to_str(self.construct_scalar(node));value_s=value_so.replace(_D,'').lower();sign=+1
		if value_s[0]==_J:sign=-1
		if value_s[0]in _M:m_sign=value_s[0];value_s=value_s[1:]
		if value_s==_p:return sign*self.inf_value
		if value_s==_q:return self.nan_value
		if self.resolver.processing_version!=(1,2)and _G in value_s:
			digits=[float(part)for part in value_s.split(_G)];digits.reverse();base=1;value=0.0
			for digit in digits:value+=digit*base;base*=60
			return sign*value
		if _N in value_s:
			try:mantissa,exponent=value_so.split(_N);exp=_N
			except ValueError:mantissa,exponent=value_so.split(A);exp=A
			if self.resolver.processing_version!=(1,2):
				if _H not in mantissa:warnings.warn(MantissaNoDotYAML1_1Warning(node,value_so))
			lead0=leading_zeros(mantissa);width=len(mantissa);prec=mantissa.find(_H)
			if m_sign:width-=1
			e_width=len(exponent);e_sign=exponent[0]in _M;return ScalarFloat(sign*float(value_s),width=width,prec=prec,m_sign=m_sign,m_lead0=lead0,exp=exp,e_width=e_width,e_sign=e_sign,anchor=node.anchor)
		width=len(value_so);prec=value_so.index(_H);lead0=leading_zeros(value_so);return ScalarFloat(sign*float(value_s),width=width,prec=prec,m_sign=m_sign,m_lead0=lead0,anchor=node.anchor)
	def construct_yaml_str(self,node):
		value=self.construct_scalar(node)
		if isinstance(value,ScalarString):return value
		if PY3:return value
		try:return value.encode(_S)
		except AttributeError:return value
		except UnicodeEncodeError:return value
	def construct_rt_sequence(self,node,seqtyp,deep=_B):
		if not isinstance(node,SequenceNode):raise ConstructorError(_A,_A,_h%node.id,node.start_mark)
		ret_val=[]
		if node.comment:
			seqtyp._yaml_add_comment(node.comment[:2])
			if len(node.comment)>2:seqtyp.yaml_end_comment_extend(node.comment[2],clear=_C)
		if node.anchor:
			from dynaconf.vendor.ruamel.yaml.serializer import templated_id
			if not templated_id(node.anchor):seqtyp.yaml_set_anchor(node.anchor)
		for (idx,child) in enumerate(node.value):
			if child.comment:seqtyp._yaml_add_comment(child.comment,key=idx);child.comment=_A
			ret_val.append(self.construct_object(child,deep=deep));seqtyp._yaml_set_idx_line_col(idx,[child.start_mark.line,child.start_mark.column])
		return ret_val
	def flatten_mapping(self,node):
		def constructed(value_node):
			if value_node in self.constructed_objects:value=self.constructed_objects[value_node]
			else:value=self.construct_object(value_node,deep=_B)
			return value
		merge_map_list=[];index=0
		while index<len(node.value):
			key_node,value_node=node.value[index]
			if key_node.tag==_k:
				if merge_map_list:
					if self.allow_duplicate_keys:del node.value[index];index+=1;continue
					args=[_E,node.start_mark,_Y.format(key_node.value),key_node.start_mark,_l,_m]
					if self.allow_duplicate_keys is _A:warnings.warn(DuplicateKeyFutureWarning(*args))
					else:raise DuplicateKeyError(*args)
				del node.value[index]
				if isinstance(value_node,MappingNode):merge_map_list.append((index,constructed(value_node)))
				elif isinstance(value_node,SequenceNode):
					for subnode in value_node.value:
						if not isinstance(subnode,MappingNode):raise ConstructorError(_E,node.start_mark,_n%subnode.id,subnode.start_mark)
						merge_map_list.append((index,constructed(subnode)))
				else:raise ConstructorError(_E,node.start_mark,_o%value_node.id,value_node.start_mark)
			elif key_node.tag==_Z:key_node.tag=_R;index+=1
			else:index+=1
		return merge_map_list
	def _sentinel(self):0
	def construct_mapping(self,node,maptyp,deep=_B):
		if not isinstance(node,MappingNode):raise ConstructorError(_A,_A,_P%node.id,node.start_mark)
		merge_map=self.flatten_mapping(node)
		if node.comment:
			maptyp._yaml_add_comment(node.comment[:2])
			if len(node.comment)>2:maptyp.yaml_end_comment_extend(node.comment[2],clear=_C)
		if node.anchor:
			from dynaconf.vendor.ruamel.yaml.serializer import templated_id
			if not templated_id(node.anchor):maptyp.yaml_set_anchor(node.anchor)
		last_key,last_value=_A,self._sentinel
		for (key_node,value_node) in node.value:
			key=self.construct_object(key_node,deep=_C)
			if not isinstance(key,Hashable):
				if isinstance(key,MutableSequence):
					key_s=CommentedKeySeq(key)
					if key_node.flow_style is _C:key_s.fa.set_flow_style()
					elif key_node.flow_style is _B:key_s.fa.set_block_style()
					key=key_s
				elif isinstance(key,MutableMapping):
					key_m=CommentedKeyMap(key)
					if key_node.flow_style is _C:key_m.fa.set_flow_style()
					elif key_node.flow_style is _B:key_m.fa.set_block_style()
					key=key_m
			if PY2:
				try:hash(key)
				except TypeError as exc:raise ConstructorError(_E,node.start_mark,_W%exc,key_node.start_mark)
			elif not isinstance(key,Hashable):raise ConstructorError(_E,node.start_mark,_X,key_node.start_mark)
			value=self.construct_object(value_node,deep=deep)
			if self.check_mapping_key(node,key_node,maptyp,key,value):
				if key_node.comment and len(key_node.comment)>4 and key_node.comment[4]:
					if last_value is _A:key_node.comment[0]=key_node.comment.pop(4);maptyp._yaml_add_comment(key_node.comment,value=last_key)
					else:key_node.comment[2]=key_node.comment.pop(4);maptyp._yaml_add_comment(key_node.comment,key=key)
					key_node.comment=_A
				if key_node.comment:maptyp._yaml_add_comment(key_node.comment,key=key)
				if value_node.comment:maptyp._yaml_add_comment(value_node.comment,value=key)
				maptyp._yaml_set_kv_line_col(key,[key_node.start_mark.line,key_node.start_mark.column,value_node.start_mark.line,value_node.start_mark.column]);maptyp[key]=value;last_key,last_value=key,value
		if merge_map:maptyp.add_yaml_merge(merge_map)
	def construct_setting(self,node,typ,deep=_B):
		if not isinstance(node,MappingNode):raise ConstructorError(_A,_A,_P%node.id,node.start_mark)
		if node.comment:
			typ._yaml_add_comment(node.comment[:2])
			if len(node.comment)>2:typ.yaml_end_comment_extend(node.comment[2],clear=_C)
		if node.anchor:
			from dynaconf.vendor.ruamel.yaml.serializer import templated_id
			if not templated_id(node.anchor):typ.yaml_set_anchor(node.anchor)
		for (key_node,value_node) in node.value:
			key=self.construct_object(key_node,deep=_C)
			if not isinstance(key,Hashable):
				if isinstance(key,list):key=tuple(key)
			if PY2:
				try:hash(key)
				except TypeError as exc:raise ConstructorError(_E,node.start_mark,_W%exc,key_node.start_mark)
			elif not isinstance(key,Hashable):raise ConstructorError(_E,node.start_mark,_X,key_node.start_mark)
			value=self.construct_object(value_node,deep=deep);self.check_set_key(node,key_node,typ,key)
			if key_node.comment:typ._yaml_add_comment(key_node.comment,key=key)
			if value_node.comment:typ._yaml_add_comment(value_node.comment,value=key)
			typ.add(key)
	def construct_yaml_seq(self,node):
		data=CommentedSeq();data._yaml_set_line_col(node.start_mark.line,node.start_mark.column)
		if node.comment:data._yaml_add_comment(node.comment)
		yield data;data.extend(self.construct_rt_sequence(node,data));self.set_collection_style(data,node)
	def construct_yaml_map(self,node):data=CommentedMap();data._yaml_set_line_col(node.start_mark.line,node.start_mark.column);yield data;self.construct_mapping(node,data,deep=_C);self.set_collection_style(data,node)
	def set_collection_style(self,data,node):
		if len(data)==0:return
		if node.flow_style is _C:data.fa.set_flow_style()
		elif node.flow_style is _B:data.fa.set_block_style()
	def construct_yaml_object(self,node,cls):
		data=cls.__new__(cls);yield data
		if hasattr(data,_V):state=SafeConstructor.construct_mapping(self,node,deep=_C);data.__setstate__(state)
		else:state=SafeConstructor.construct_mapping(self,node);data.__dict__.update(state)
	def construct_yaml_omap(self,node):
		omap=CommentedOrderedMap();omap._yaml_set_line_col(node.start_mark.line,node.start_mark.column)
		if node.flow_style is _C:omap.fa.set_flow_style()
		elif node.flow_style is _B:omap.fa.set_block_style()
		yield omap
		if node.comment:
			omap._yaml_add_comment(node.comment[:2])
			if len(node.comment)>2:omap.yaml_end_comment_extend(node.comment[2],clear=_C)
		if not isinstance(node,SequenceNode):raise ConstructorError(_L,node.start_mark,_b%node.id,node.start_mark)
		for subnode in node.value:
			if not isinstance(subnode,MappingNode):raise ConstructorError(_L,node.start_mark,_c%subnode.id,subnode.start_mark)
			if len(subnode.value)!=1:raise ConstructorError(_L,node.start_mark,_d%len(subnode.value),subnode.start_mark)
			key_node,value_node=subnode.value[0];key=self.construct_object(key_node);assert key not in omap;value=self.construct_object(value_node)
			if key_node.comment:omap._yaml_add_comment(key_node.comment,key=key)
			if subnode.comment:omap._yaml_add_comment(subnode.comment,key=key)
			if value_node.comment:omap._yaml_add_comment(value_node.comment,value=key)
			omap[key]=value
	def construct_yaml_set(self,node):data=CommentedSet();data._yaml_set_line_col(node.start_mark.line,node.start_mark.column);yield data;self.construct_setting(node,data)
	def construct_undefined(self,node):
		try:
			if isinstance(node,MappingNode):
				data=CommentedMap();data._yaml_set_line_col(node.start_mark.line,node.start_mark.column)
				if node.flow_style is _C:data.fa.set_flow_style()
				elif node.flow_style is _B:data.fa.set_block_style()
				data.yaml_set_tag(node.tag);yield data
				if node.anchor:data.yaml_set_anchor(node.anchor)
				self.construct_mapping(node,data);return
			elif isinstance(node,ScalarNode):
				data2=TaggedScalar();data2.value=self.construct_scalar(node);data2.style=node.style;data2.yaml_set_tag(node.tag);yield data2
				if node.anchor:data2.yaml_set_anchor(node.anchor,always_dump=_C)
				return
			elif isinstance(node,SequenceNode):
				data3=CommentedSeq();data3._yaml_set_line_col(node.start_mark.line,node.start_mark.column)
				if node.flow_style is _C:data3.fa.set_flow_style()
				elif node.flow_style is _B:data3.fa.set_block_style()
				data3.yaml_set_tag(node.tag);yield data3
				if node.anchor:data3.yaml_set_anchor(node.anchor)
				data3.extend(self.construct_sequence(node));return
		except:pass
		raise ConstructorError(_A,_A,_z%utf8(node.tag),node.start_mark)
	def construct_yaml_timestamp(self,node,values=_A):
		B='t';A='tz'
		try:match=self.timestamp_regexp.match(node.value)
		except TypeError:match=_A
		if match is _A:raise ConstructorError(_A,_A,_t.format(node.value),node.start_mark)
		values=match.groupdict()
		if not values[_T]:return SafeConstructor.construct_yaml_timestamp(self,node,values)
		for part in [B,_K,_U,_O]:
			if values[part]:break
		else:return SafeConstructor.construct_yaml_timestamp(self,node,values)
		year=int(values[_u]);month=int(values[_v]);day=int(values[_w]);hour=int(values[_T]);minute=int(values[_x]);second=int(values[_y]);fraction=0
		if values[_I]:
			fraction_s=values[_I][:6]
			while len(fraction_s)<6:fraction_s+=_F
			fraction=int(fraction_s)
			if len(values[_I])>6 and int(values[_I][6])>4:fraction+=1
		delta=_A
		if values[_K]:
			tz_hour=int(values[_U]);minutes=values[_O];tz_minute=int(minutes)if minutes else 0;delta=datetime.timedelta(hours=tz_hour,minutes=tz_minute)
			if values[_K]==_J:delta=-delta
		if delta:
			dt=datetime.datetime(year,month,day,hour,minute);dt-=delta;data=TimeStamp(dt.year,dt.month,dt.day,dt.hour,dt.minute,second,fraction);data._yaml['delta']=delta;tz=values[_K]+values[_U]
			if values[_O]:tz+=_G+values[_O]
			data._yaml[A]=tz
		else:
			data=TimeStamp(year,month,day,hour,minute,second,fraction)
			if values[A]:data._yaml[A]=values[A]
		if values[B]:data._yaml[B]=_C
		return data
	def construct_yaml_bool(self,node):
		b=SafeConstructor.construct_yaml_bool(self,node)
		if node.anchor:return ScalarBoolean(b,anchor=node.anchor)
		return b
RoundTripConstructor.add_constructor(_A0,RoundTripConstructor.construct_yaml_null)
RoundTripConstructor.add_constructor(_A1,RoundTripConstructor.construct_yaml_bool)
RoundTripConstructor.add_constructor(_A2,RoundTripConstructor.construct_yaml_int)
RoundTripConstructor.add_constructor(_A3,RoundTripConstructor.construct_yaml_float)
RoundTripConstructor.add_constructor(_A4,RoundTripConstructor.construct_yaml_binary)
RoundTripConstructor.add_constructor(_A5,RoundTripConstructor.construct_yaml_timestamp)
RoundTripConstructor.add_constructor(_A6,RoundTripConstructor.construct_yaml_omap)
RoundTripConstructor.add_constructor(_A7,RoundTripConstructor.construct_yaml_pairs)
RoundTripConstructor.add_constructor(_A8,RoundTripConstructor.construct_yaml_set)
RoundTripConstructor.add_constructor(_R,RoundTripConstructor.construct_yaml_str)
RoundTripConstructor.add_constructor(_A9,RoundTripConstructor.construct_yaml_seq)
RoundTripConstructor.add_constructor(_AA,RoundTripConstructor.construct_yaml_map)
RoundTripConstructor.add_constructor(_A,RoundTripConstructor.construct_undefined)


from __future__ import print_function
_D='RUAMELDEBUG'
_C=True
_B=False
_A=None
import sys,os,types,traceback
from abc import abstractmethod
if _B:from typing import Any,Dict,Optional,List,Union,BinaryIO,IO,Text,Tuple,Optional
_DEFAULT_YAML_VERSION=1,2
try:from ruamel.ordereddict import ordereddict
except:
	try:from collections import OrderedDict
	except ImportError:from ordereddict import OrderedDict
	class ordereddict(OrderedDict):
		if not hasattr(OrderedDict,'insert'):
			def insert(A,pos,key,value):
				C=value
				if pos>=len(A):A[key]=C;return
				B=ordereddict();B.update(A)
				for E in B:del A[E]
				for (F,D) in enumerate(B):
					if pos==F:A[key]=C
					A[D]=B[D]
PY2=sys.version_info[0]==2
PY3=sys.version_info[0]==3
if PY3:
	def utf8(s):return s
	def to_str(s):return s
	def to_unicode(s):return s
else:
	if _B:unicode=str
	def utf8(s):return s.encode('utf-8')
	def to_str(s):return str(s)
	def to_unicode(s):return unicode(s)
if PY3:string_types=str;integer_types=int;class_types=type;text_type=str;binary_type=bytes;MAXSIZE=sys.maxsize;unichr=chr;import io;StringIO=io.StringIO;BytesIO=io.BytesIO;no_limit_int=int;from collections.abc import Hashable,MutableSequence,MutableMapping,Mapping
else:string_types=basestring;integer_types=int,long;class_types=type,types.ClassType;text_type=unicode;binary_type=str;unichr=unichr;from StringIO import StringIO as _StringIO;StringIO=_StringIO;import cStringIO;BytesIO=cStringIO.StringIO;no_limit_int=long;from collections import Hashable,MutableSequence,MutableMapping,Mapping
if _B:StreamType=Any;StreamTextType=StreamType;VersionType=Union[(List[int],str,Tuple[(int,int)])]
if PY3:builtins_module='builtins'
else:builtins_module='__builtin__'
UNICODE_SIZE=4 if sys.maxunicode>65535 else 2
def with_metaclass(meta,*A):return meta('NewBase',A,{})
DBG_TOKEN=1
DBG_EVENT=2
DBG_NODE=4
_debug=_A
if _D in os.environ:
	_debugx=os.environ.get(_D)
	if _debugx is _A:_debug=0
	else:_debug=int(_debugx)
if bool(_debug):
	class ObjectCounter:
		def __init__(A):A.map={}
		def __call__(A,k):A.map[k]=A.map.get(k,0)+1
		def dump(A):
			for B in sorted(A.map):sys.stdout.write('{} -> {}'.format(B,A.map[B]))
	object_counter=ObjectCounter()
def dbg(val=_A):
	global _debug
	if _debug is _A:
		A=os.environ.get('YAMLDEBUG')
		if A is _A:_debug=0
		else:_debug=int(A)
	if val is _A:return _debug
	return _debug&val
class Nprint:
	def __init__(A,file_name=_A):A._max_print=_A;A._count=_A;A._file_name=file_name
	def __call__(A,*E,**F):
		if not bool(_debug):return
		B=sys.stdout if A._file_name is _A else open(A._file_name,'a');C=print;D=F.copy();D['file']=B;C(*E,**D);B.flush()
		if A._max_print is not _A:
			if A._count is _A:A._count=A._max_print
			A._count-=1
			if A._count==0:C('forced exit\n');traceback.print_stack();B.flush();sys.exit(0)
		if A._file_name:B.close()
	def set_max_print(A,i):A._max_print=i;A._count=_A
nprint=Nprint()
nprintf=Nprint('/var/tmp/ruamel.yaml.log')
def check_namespace_char(ch):
	A=ch
	if'!'<=A<='~':return _C
	if'\xa0'<=A<='\ud7ff':return _C
	if'\ue000'<=A<='�'and A!='\ufeff':return _C
	if'𐀀'<=A<='\U0010ffff':return _C
	return _B
def check_anchorname_char(ch):
	if ch in',[]{}':return _B
	return check_namespace_char(ch)
def version_tnf(t1,t2=_A):
	from dynaconf.vendor.ruamel.yaml import version_info as A
	if A<t1:return _C
	if t2 is not _A and A<t2:return _A
	return _B
class MutableSliceableSequence(MutableSequence):
	__slots__=()
	def __getitem__(A,index):
		B=index
		if not isinstance(B,slice):return A.__getsingleitem__(B)
		return type(A)([A[C]for C in range(*B.indices(len(A)))])
	def __setitem__(C,index,value):
		B=value;A=index
		if not isinstance(A,slice):return C.__setsingleitem__(A,B)
		assert iter(B)
		if A.step is _A:
			del C[A.start:A.stop]
			for F in reversed(B):C.insert(0 if A.start is _A else A.start,F)
		else:
			D=A.indices(len(C));E=(D[1]-D[0]-1)//D[2]+1
			if E<len(B):raise TypeError('too many elements in value {} < {}'.format(E,len(B)))
			elif E>len(B):raise TypeError('not enough elements in value {} > {}'.format(E,len(B)))
			for (G,H) in enumerate(range(*D)):C[H]=B[G]
	def __delitem__(A,index):
		B=index
		if not isinstance(B,slice):return A.__delsingleitem__(B)
		for C in reversed(range(*B.indices(len(A)))):del A[C]
	@abstractmethod
	def __getsingleitem__(self,index):raise IndexError
	@abstractmethod
	def __setsingleitem__(self,index,value):raise IndexError
	@abstractmethod
	def __delsingleitem__(self,index):raise IndexError